import typing
import logging
import lxml
import sys
import traceback


class PresalyticsBaseException(Exception):
    def __init__(self, *args, **kwargs):
        default_message = "An Error occur in the presalytics module.  Consult log for stacktrace."
        message = args[0]
        if message:
            self.message = message
        else:
            self.message = default_message
        if len(message) > 0:
            logger = logging.getLogger('presalytics.lib.exceptions')
            logger.error(message)
        if not (args or kwargs):
            args = (default_message,)
            super().__init__(*args, **kwargs)


class MissingConfigException(PresalyticsBaseException):
    def __init__(self, message):
        if message is None:
            message = "Configuration file missing.  Please please a config.ini file in working directory"
        super().__init__(message)


class MisConfiguredTokenException(PresalyticsBaseException):
    def __init__(self):
        message = "Authenication token is missing or malformed.  Please acquire new token and try again."
        super().__init__(message)


class InvalidTokenException(PresalyticsBaseException):
    def __init__(self, message=None):
        if not message:
            message = "This token is invalid.  Please login again to get a new token."
        super().__init__(message)


class LoginTimeout(PresalyticsBaseException):
    def __init__(self):
        message = "User took too long to login on website to obtain token.  Please try again."
        super().__init__(message)


class ValidationError(PresalyticsBaseException):
    def __init__(self, message):
        if not message:
            message = "Validation Error occured"
        super().__init__(message)


class ApiError(PresalyticsBaseException):
    def __init__(self, message=None, status_code=None):
        self.status_code = status_code
        if not message:
            if not status_code:
                message = "A error occured while commuicating with the presalytics API.  Please Check your configuration values and try again."
            else:
                message = "Status Code: {0}".format(status_code)
        else:
            if status_code:
                try:
                    message = message.decode('utf-8')
                except Exception:
                    pass
                message = str(message) + ".  Status Code: {0}".format(status_code)
        super().__init__(message)


class InvalidConfigurationError(PresalyticsBaseException):
    def __init__(self, message=None):
        if not message:
            message = "One of the input parameters to your component is incorrectly defined (e.g., a typo).  Please re-check and try again."
        super().__init__(message)


class RegistryError(PresalyticsBaseException):
     def __init__(self, registry, message=None):
        if not message:
            message = "The was an unknown error in inside the registry"
        message = "{0} Error: ".format(registry.__class__.__name__) + message
        super().__init__(message)


class InvalidArgumentException(PresalyticsBaseException):
    def __init__(self, message=None):
        if not message:
            message = "One of the arguments supplied to this method is invalid."
        super().__init__(message)

class ApiException(PresalyticsBaseException):
    def __init__(self, default_exception=None):
        if default_exception is not None:
            _attrs = [a for a in dir(default_exception) if not a.startswith('__')]
            for key in _attrs:
                setattr(self, key, getattr(default_exception, key))

    def __str__(self):
        """Custom error messages for exception"""
        try:
            error_message = "({0})\nReason: {1}\n".format(self.status, self.reason)

            if self.body:
                error_message += "HTTP response body: {0}\n".format(self.body)

            return error_message
        except Exception:
            return "An unknown error occured.  Please set default_exception to learn more."


class RenderExceptionHandler(object):
    def __init__(self, exception: Exception, target_type="widget", traceback=None):
        self.exception = exception
        self.target_type = target_type
        self.exception_type = self.exception.__class__.__name__
        self.line_no = None

        try: 
            first_frame = self.get_source_frame(traceback)
            self.source_module = first_frame.tb_frame.f_globals['__name__']
            self.line_no = first_frame.tb_lineno
        except Exception:
            self.source_module = "unidentified"
            self.line_no = "unknown"
    
        if isinstance(self.exception, PresalyticsBaseException):
            self.message = self.exception.message
        else:
            try:
                self.message = self.exception.args[0]
            except (AttributeError, KeyError):
                self.message = "No message was included with this exception."

    def get_source_frame(self, tb):
        next = tb.tb_next
        if next:
            return self.get_source_frame(next)
        else:
            return tb
        

    def render_exception(self):
        container = lxml.html.Element("div", {
            'class': "exception-container"
        })
        header = lxml.html.Element("h3")
        message = lxml.html.Element("p")
        header.text = "Oops! A rendering error occured."
        message.text = "This {} could not be rendered.  Please see the information below to help you diagnose the problem".format(self.target_type)

        _type = lxml.html.Element("p")
        _type.text = "Exception Type: {}".format(self.exception_type)

        exception_message = lxml.html.Element("p")
        exception_message.text = "Exception message: {}".format(self.message)
        source = lxml.html.Element("p")
        source.text = "Error at line number: {0} in module {1}".format(self.line_no, self.source_module)
        note = lxml.html.Element("p")
        note.text = "If you have trouble understainding this error message, try building your story using " \
            "with the presalytics.Revealer's `present()` method.  If should give you more thorough error logging."
        
        container.extend([header, message, _type, exception_message, source, note])
        return lxml.html.tostring(container).decode('utf-8')
    
    def to_html(self):
        return self.render_exception()
         


# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class SharedFillMapDetails(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'fill_type_id': 'int',
        'solid_fill': 'SharedSolidFillsDetails',
        'gradient_fill': 'SharedGradientFillsDetails',
        'image_fill': 'SharedImageFillsDetails',
        'shape': 'SlideShapesDetails',
        'shape_id': 'str',
        'connector': 'SlideConnectorDetails',
        'connector_id': 'str',
        'effect_attribute_id': 'str',
        'effect_attribute': 'SharedEffectAttributesDetails',
        'table_cell_id': 'str',
        'table_cell': 'TableCellsDetails',
        'theme_background_fill_id': 'str',
        'theme_background_fill': 'ThemeBackgroundFillsDetails',
        'theme_fill_id': 'str',
        'theme_fill': 'ThemeFillsDetails',
        'id': 'str',
        'date_created': 'datetime',
        'user_created': 'str',
        'date_modified': 'datetime',
        'user_modified': 'str'
    }

    attribute_map = {
        'fill_type_id': 'fillTypeId',
        'solid_fill': 'solidFill',
        'gradient_fill': 'gradientFill',
        'image_fill': 'imageFill',
        'shape': 'shape',
        'shape_id': 'shapeId',
        'connector': 'connector',
        'connector_id': 'connectorId',
        'effect_attribute_id': 'effectAttributeId',
        'effect_attribute': 'effectAttribute',
        'table_cell_id': 'tableCellId',
        'table_cell': 'tableCell',
        'theme_background_fill_id': 'themeBackgroundFillId',
        'theme_background_fill': 'themeBackgroundFill',
        'theme_fill_id': 'themeFillId',
        'theme_fill': 'themeFill',
        'id': 'id',
        'date_created': 'dateCreated',
        'user_created': 'userCreated',
        'date_modified': 'dateModified',
        'user_modified': 'userModified'
    }

    def __init__(self, fill_type_id=None, solid_fill=None, gradient_fill=None, image_fill=None, shape=None, shape_id=None, connector=None, connector_id=None, effect_attribute_id=None, effect_attribute=None, table_cell_id=None, table_cell=None, theme_background_fill_id=None, theme_background_fill=None, theme_fill_id=None, theme_fill=None, id=None, date_created=None, user_created=None, date_modified=None, user_modified=None):  # noqa: E501
        """SharedFillMapDetails - a model defined in OpenAPI"""  # noqa: E501

        self._fill_type_id = None
        self._solid_fill = None
        self._gradient_fill = None
        self._image_fill = None
        self._shape = None
        self._shape_id = None
        self._connector = None
        self._connector_id = None
        self._effect_attribute_id = None
        self._effect_attribute = None
        self._table_cell_id = None
        self._table_cell = None
        self._theme_background_fill_id = None
        self._theme_background_fill = None
        self._theme_fill_id = None
        self._theme_fill = None
        self._id = None
        self._date_created = None
        self._user_created = None
        self._date_modified = None
        self._user_modified = None
        self.discriminator = None

        if fill_type_id is not None:
            self.fill_type_id = fill_type_id
        if solid_fill is not None:
            self.solid_fill = solid_fill
        if gradient_fill is not None:
            self.gradient_fill = gradient_fill
        if image_fill is not None:
            self.image_fill = image_fill
        if shape is not None:
            self.shape = shape
        self.shape_id = shape_id
        if connector is not None:
            self.connector = connector
        self.connector_id = connector_id
        self.effect_attribute_id = effect_attribute_id
        if effect_attribute is not None:
            self.effect_attribute = effect_attribute
        self.table_cell_id = table_cell_id
        if table_cell is not None:
            self.table_cell = table_cell
        self.theme_background_fill_id = theme_background_fill_id
        if theme_background_fill is not None:
            self.theme_background_fill = theme_background_fill
        self.theme_fill_id = theme_fill_id
        if theme_fill is not None:
            self.theme_fill = theme_fill
        if id is not None:
            self.id = id
        if date_created is not None:
            self.date_created = date_created
        if user_created is not None:
            self.user_created = user_created
        if date_modified is not None:
            self.date_modified = date_modified
        if user_modified is not None:
            self.user_modified = user_modified

    @property
    def fill_type_id(self):
        """Gets the fill_type_id of this SharedFillMapDetails.  # noqa: E501


        :return: The fill_type_id of this SharedFillMapDetails.  # noqa: E501
        :rtype: int
        """
        return self._fill_type_id

    @fill_type_id.setter
    def fill_type_id(self, fill_type_id):
        """Sets the fill_type_id of this SharedFillMapDetails.


        :param fill_type_id: The fill_type_id of this SharedFillMapDetails.  # noqa: E501
        :type: int
        """

        self._fill_type_id = fill_type_id

    @property
    def solid_fill(self):
        """Gets the solid_fill of this SharedFillMapDetails.  # noqa: E501


        :return: The solid_fill of this SharedFillMapDetails.  # noqa: E501
        :rtype: SharedSolidFillsDetails
        """
        return self._solid_fill

    @solid_fill.setter
    def solid_fill(self, solid_fill):
        """Sets the solid_fill of this SharedFillMapDetails.


        :param solid_fill: The solid_fill of this SharedFillMapDetails.  # noqa: E501
        :type: SharedSolidFillsDetails
        """

        self._solid_fill = solid_fill

    @property
    def gradient_fill(self):
        """Gets the gradient_fill of this SharedFillMapDetails.  # noqa: E501


        :return: The gradient_fill of this SharedFillMapDetails.  # noqa: E501
        :rtype: SharedGradientFillsDetails
        """
        return self._gradient_fill

    @gradient_fill.setter
    def gradient_fill(self, gradient_fill):
        """Sets the gradient_fill of this SharedFillMapDetails.


        :param gradient_fill: The gradient_fill of this SharedFillMapDetails.  # noqa: E501
        :type: SharedGradientFillsDetails
        """

        self._gradient_fill = gradient_fill

    @property
    def image_fill(self):
        """Gets the image_fill of this SharedFillMapDetails.  # noqa: E501


        :return: The image_fill of this SharedFillMapDetails.  # noqa: E501
        :rtype: SharedImageFillsDetails
        """
        return self._image_fill

    @image_fill.setter
    def image_fill(self, image_fill):
        """Sets the image_fill of this SharedFillMapDetails.


        :param image_fill: The image_fill of this SharedFillMapDetails.  # noqa: E501
        :type: SharedImageFillsDetails
        """

        self._image_fill = image_fill

    @property
    def shape(self):
        """Gets the shape of this SharedFillMapDetails.  # noqa: E501


        :return: The shape of this SharedFillMapDetails.  # noqa: E501
        :rtype: SlideShapesDetails
        """
        return self._shape

    @shape.setter
    def shape(self, shape):
        """Sets the shape of this SharedFillMapDetails.


        :param shape: The shape of this SharedFillMapDetails.  # noqa: E501
        :type: SlideShapesDetails
        """

        self._shape = shape

    @property
    def shape_id(self):
        """Gets the shape_id of this SharedFillMapDetails.  # noqa: E501


        :return: The shape_id of this SharedFillMapDetails.  # noqa: E501
        :rtype: str
        """
        return self._shape_id

    @shape_id.setter
    def shape_id(self, shape_id):
        """Sets the shape_id of this SharedFillMapDetails.


        :param shape_id: The shape_id of this SharedFillMapDetails.  # noqa: E501
        :type: str
        """

        self._shape_id = shape_id

    @property
    def connector(self):
        """Gets the connector of this SharedFillMapDetails.  # noqa: E501


        :return: The connector of this SharedFillMapDetails.  # noqa: E501
        :rtype: SlideConnectorDetails
        """
        return self._connector

    @connector.setter
    def connector(self, connector):
        """Sets the connector of this SharedFillMapDetails.


        :param connector: The connector of this SharedFillMapDetails.  # noqa: E501
        :type: SlideConnectorDetails
        """

        self._connector = connector

    @property
    def connector_id(self):
        """Gets the connector_id of this SharedFillMapDetails.  # noqa: E501


        :return: The connector_id of this SharedFillMapDetails.  # noqa: E501
        :rtype: str
        """
        return self._connector_id

    @connector_id.setter
    def connector_id(self, connector_id):
        """Sets the connector_id of this SharedFillMapDetails.


        :param connector_id: The connector_id of this SharedFillMapDetails.  # noqa: E501
        :type: str
        """

        self._connector_id = connector_id

    @property
    def effect_attribute_id(self):
        """Gets the effect_attribute_id of this SharedFillMapDetails.  # noqa: E501


        :return: The effect_attribute_id of this SharedFillMapDetails.  # noqa: E501
        :rtype: str
        """
        return self._effect_attribute_id

    @effect_attribute_id.setter
    def effect_attribute_id(self, effect_attribute_id):
        """Sets the effect_attribute_id of this SharedFillMapDetails.


        :param effect_attribute_id: The effect_attribute_id of this SharedFillMapDetails.  # noqa: E501
        :type: str
        """

        self._effect_attribute_id = effect_attribute_id

    @property
    def effect_attribute(self):
        """Gets the effect_attribute of this SharedFillMapDetails.  # noqa: E501


        :return: The effect_attribute of this SharedFillMapDetails.  # noqa: E501
        :rtype: SharedEffectAttributesDetails
        """
        return self._effect_attribute

    @effect_attribute.setter
    def effect_attribute(self, effect_attribute):
        """Sets the effect_attribute of this SharedFillMapDetails.


        :param effect_attribute: The effect_attribute of this SharedFillMapDetails.  # noqa: E501
        :type: SharedEffectAttributesDetails
        """

        self._effect_attribute = effect_attribute

    @property
    def table_cell_id(self):
        """Gets the table_cell_id of this SharedFillMapDetails.  # noqa: E501


        :return: The table_cell_id of this SharedFillMapDetails.  # noqa: E501
        :rtype: str
        """
        return self._table_cell_id

    @table_cell_id.setter
    def table_cell_id(self, table_cell_id):
        """Sets the table_cell_id of this SharedFillMapDetails.


        :param table_cell_id: The table_cell_id of this SharedFillMapDetails.  # noqa: E501
        :type: str
        """

        self._table_cell_id = table_cell_id

    @property
    def table_cell(self):
        """Gets the table_cell of this SharedFillMapDetails.  # noqa: E501


        :return: The table_cell of this SharedFillMapDetails.  # noqa: E501
        :rtype: TableCellsDetails
        """
        return self._table_cell

    @table_cell.setter
    def table_cell(self, table_cell):
        """Sets the table_cell of this SharedFillMapDetails.


        :param table_cell: The table_cell of this SharedFillMapDetails.  # noqa: E501
        :type: TableCellsDetails
        """

        self._table_cell = table_cell

    @property
    def theme_background_fill_id(self):
        """Gets the theme_background_fill_id of this SharedFillMapDetails.  # noqa: E501


        :return: The theme_background_fill_id of this SharedFillMapDetails.  # noqa: E501
        :rtype: str
        """
        return self._theme_background_fill_id

    @theme_background_fill_id.setter
    def theme_background_fill_id(self, theme_background_fill_id):
        """Sets the theme_background_fill_id of this SharedFillMapDetails.


        :param theme_background_fill_id: The theme_background_fill_id of this SharedFillMapDetails.  # noqa: E501
        :type: str
        """

        self._theme_background_fill_id = theme_background_fill_id

    @property
    def theme_background_fill(self):
        """Gets the theme_background_fill of this SharedFillMapDetails.  # noqa: E501


        :return: The theme_background_fill of this SharedFillMapDetails.  # noqa: E501
        :rtype: ThemeBackgroundFillsDetails
        """
        return self._theme_background_fill

    @theme_background_fill.setter
    def theme_background_fill(self, theme_background_fill):
        """Sets the theme_background_fill of this SharedFillMapDetails.


        :param theme_background_fill: The theme_background_fill of this SharedFillMapDetails.  # noqa: E501
        :type: ThemeBackgroundFillsDetails
        """

        self._theme_background_fill = theme_background_fill

    @property
    def theme_fill_id(self):
        """Gets the theme_fill_id of this SharedFillMapDetails.  # noqa: E501


        :return: The theme_fill_id of this SharedFillMapDetails.  # noqa: E501
        :rtype: str
        """
        return self._theme_fill_id

    @theme_fill_id.setter
    def theme_fill_id(self, theme_fill_id):
        """Sets the theme_fill_id of this SharedFillMapDetails.


        :param theme_fill_id: The theme_fill_id of this SharedFillMapDetails.  # noqa: E501
        :type: str
        """

        self._theme_fill_id = theme_fill_id

    @property
    def theme_fill(self):
        """Gets the theme_fill of this SharedFillMapDetails.  # noqa: E501


        :return: The theme_fill of this SharedFillMapDetails.  # noqa: E501
        :rtype: ThemeFillsDetails
        """
        return self._theme_fill

    @theme_fill.setter
    def theme_fill(self, theme_fill):
        """Sets the theme_fill of this SharedFillMapDetails.


        :param theme_fill: The theme_fill of this SharedFillMapDetails.  # noqa: E501
        :type: ThemeFillsDetails
        """

        self._theme_fill = theme_fill

    @property
    def id(self):
        """Gets the id of this SharedFillMapDetails.  # noqa: E501


        :return: The id of this SharedFillMapDetails.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SharedFillMapDetails.


        :param id: The id of this SharedFillMapDetails.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def date_created(self):
        """Gets the date_created of this SharedFillMapDetails.  # noqa: E501


        :return: The date_created of this SharedFillMapDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_created

    @date_created.setter
    def date_created(self, date_created):
        """Sets the date_created of this SharedFillMapDetails.


        :param date_created: The date_created of this SharedFillMapDetails.  # noqa: E501
        :type: datetime
        """

        self._date_created = date_created

    @property
    def user_created(self):
        """Gets the user_created of this SharedFillMapDetails.  # noqa: E501


        :return: The user_created of this SharedFillMapDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_created

    @user_created.setter
    def user_created(self, user_created):
        """Sets the user_created of this SharedFillMapDetails.


        :param user_created: The user_created of this SharedFillMapDetails.  # noqa: E501
        :type: str
        """

        self._user_created = user_created

    @property
    def date_modified(self):
        """Gets the date_modified of this SharedFillMapDetails.  # noqa: E501


        :return: The date_modified of this SharedFillMapDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_modified

    @date_modified.setter
    def date_modified(self, date_modified):
        """Sets the date_modified of this SharedFillMapDetails.


        :param date_modified: The date_modified of this SharedFillMapDetails.  # noqa: E501
        :type: datetime
        """

        self._date_modified = date_modified

    @property
    def user_modified(self):
        """Gets the user_modified of this SharedFillMapDetails.  # noqa: E501


        :return: The user_modified of this SharedFillMapDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_modified

    @user_modified.setter
    def user_modified(self, user_modified):
        """Sets the user_modified of this SharedFillMapDetails.


        :param user_modified: The user_modified of this SharedFillMapDetails.  # noqa: E501
        :type: str
        """

        self._user_modified = user_modified

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SharedFillMapDetails):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""
# Presalytics Python Library
For more information, visit https://presalytics.io.

# Overview

The Presalytics Python Library streamlines analytic operations across analysts, executives, consultants, 
and developers.  These tools enable for a simplified workflow for analysts to rapidly generate 
client-ready presentation materials and web content that update in real-time and easily scale across your user-base.  

Our objective when building this platform is make the analyst experience as simple as possible.  
Set up time for new user should take less than half an hour, and all features are self-service and self-explanatory.  

Of course, if you have any questions or need help to get going, you can [contact us](/contact-us) or 
quickly get the help you need on our [slack channel](https://presalytics.slack.com) 
([Join Here!](https://join.slack.com/t/presalytics/shared_invite/enQtODExMjc3MDE1Nzc5LWU0ZDlhZTgwZTM3MzQ4Yzc4Nzk4Zjc0NmQ3YjgzNTEwODdlYjM0ZjFkZWI4Y2ZhNzBmOTZhMzA2MzE3YjFiZTg)).

To get going quickly, you can browse our [Getting Started](https://presalytics.io/docs/getting-started/) page and review 
some [examples](https://presalytics.io/docs/examples).

For more advanced users and developers, you can learn more about the API by reviewing 
the [service structure](https://presalytics.io/docs/how-it-works) to build a better understanding 
of the API and its [security](https://presalytics.io/docs/develpers/security) features.

# Installation

The Presalytics Python Library is avialabe in python package index, and can be installed via pip:

~~~~python
pip install presalytics
~~~~

# Contributing

Presalytics.io is on [Github](https://github.com/presalytics).  Bug reports and pull requests are strongly encouraged at
the package [repository](https://github.com/presalytics/python-client). If you encounter any problems or have any suggestions for 
the API endpoints that this libary interacts with at https://api.presalytics.io, please open an issue in the 
[API repository](https://github.com/presalytics/Presalytics-API).

# License

The Presalytics Python Library can be used in any of your applications and is covered by the MIT License.  This library
exchanges infromation with other web APIs that may be proprietary and carry their own licensing restriction.  More more
information on licensing, please contact [inquires@presalytics.io](mailto:inquires@presalytics.io).

"""
import os
import environs
import logging
import pkg_resources
import presalytics.lib
import presalytics.lib.logger
import presalytics.lib.plugins
import presalytics.lib.plugins.base
import presalytics.lib.config_loader
import presalytics.story
import presalytics.story.components

env = environs.Env()
env.read_env()

# A comma-separated list of paths to search of config.py, plugin classes, and component classes
autodiscover_paths = env.list('AUTODISCOVER_PATHS', [])

CONFIG = presalytics.lib.config_loader.load_config(additional_paths=autodiscover_paths)
"""
Nested `dict` containing runtime configuration values for the Presalytics Python Library.
Typically, these are reuseable values stored in separate file that are loaded when the 
module in imported via the `import presalytics` command.  This top-level module then
runs the `presalytics.lib.config_loader.load_config` method into load values into the
the `CONFIG` global variable.  Modules through the package use the `CONFIG` variable to
simplify their API calls store conststants for use throughout the package.  See 
`presalytics.lib.config_loader.load_config` for ways to programmatically load the
`CONFIG`.

Configuration Values
----------

USE_LOGGER : bool, optional
    Toggles whether the presalytics verbose file logger should be used.  Helpful for 
    tracing exceptions while writing code.  Default is True.

LOG_LEVEL : str, optional
    Defaults to `DEBUG`

USERNAME : str, optional
    The user's Presalytics API email/username.  This is the email address that the user uses when logging in at 
    https://login.presalytics.io.  Will be passed to instances of the `presalytics.client.api.Client` object.

PASSWORD : str, optional
    The user's Presalytics API username.  Will be passed to instances of the 
    `presalytics.client.api.Client` object.  If running in an insecure or 
    multiuser environment, leave this blank and let the `presalytics.client.api.Client`
    object handle token acquisition via browser-based login.

DELEGATE_LOGIN: bool, optional
    Defaults to False.  Indicates whether the client would redirect to a browser to 
    acquire an API token. If `DELEGATE_LOGIN` is `True`, when the `presalytics.client.api.Client` does not have 
    access to a valid API token, the client will raise a `presalytics.lib.exceptions.InvalidTokenException`.
    The default operation will automatically open a new browser tab to acquire a new token 
    via website client from the presalytics.io login page.  Putting this setting to True is
    useful for server-side development.

CACHE_TOKENS: bool, optional
    Defaults to True.  Indicates whether the `presalytics.client.api.Client` should store
    tokens in the current working directory in a file call "token.json". This should be
    set to False for mutli-user environments.

CLIENT_ID : str, optional
    For developer use. Allows developers to implement a `client_credentials` OpenID
    Connect login.  Defaults to "python-client".  

CLIENT_SECRET : str, optional
    For developer use. Allows developers to implement a `client_credentials` OpenID
    Connect login.  Defaults to None.  

VERIFY_HTTPS : bool, optional
    For developer use.  Allows for unencrypted connections.  Defaults to True.  No 
    reason to turn this to False unless you're in a complex development scenario
    and you know what you're doing.

HOSTS : dict, optional
    For developer use.  Allows API class to target hosts other than api.presalytics.io

REDIRECT_URI : string, optional
    For developer use.  Useful if implementing authorization code flow for and OpenID Connect client.
    Redirect URIs must be approved by Presalytics API devops for use in client applications.

RESERVED_NAMES: list of str, optional
    A list of filenames for *.py files in the current workspace that should be ignored by the 
    registries. 

IGNORE_PATHS: list of str, optional
    A list of paths to not to include in registry autosdiscover

BROWSER_API_HOST: dict, optional
    If present, the root url for browser-based api calls.  Each service can have an independent browser host.
    Service keys include `OOXML_AUTOMATION`, `STORY`, `DOC_CONVERTER`, and `SITE`.  May be required when services are running on a cluster that
    deletegates certificate authentication to an external service or in debug/test environments.  
    See for more info: https://developer.mozilla.org/en-US/docs/Web/Security/Mixed_content

The object can also take on values for user-defined extensions, and please consult
the documentation for those package for those vairables definition.
"""



file_logger = CONFIG.get("USE_LOGGER", True)
log_level = CONFIG.get("LOG_LEVEL", logging.DEBUG)

presalytics.lib.logger.configure_logger(log_level=log_level, file_logger=file_logger)

registry_kwargs = {
    'show_errors': False,
    'autodiscover_paths': autodiscover_paths,
    'reserved_names': CONFIG.get("RESERVED_NAMES", []),
    'ignore_paths': CONFIG.get("IGNORE_PATHS", [])
}

PLUGINS = presalytics.lib.plugins.base.PluginRegistry(**registry_kwargs)
"""
Instance of `presalytics.lib.plugins.base.PluginRegistry`.  A container listing the
Presalytics Library Plugins available and loaded in this environment. This instance is used 
by `presalytics.story.components.Renderer` subclasses (e.g., `presalytics.story.revealer.Revealer`)
to write scripts and links into stories.
"""

COMPONENTS = presalytics.story.components.ComponentRegistry(**registry_kwargs)
"""
Instance of `presalytics.story.components.ComponentRegistry`.  Registry for Library components and 
component instances.  A container listing the Presalytics Library components and instances available 
and loaded in this environment. This instance is used by `presalytics.story.components.Renderer` subclasses 
(e.g., `presalytics.story.revealer.Revealer`) to convert widgets, pages, and themes into stories.
"""

try:
    __version__ = pkg_resources.require(presalytics.__name__)[0].version
except Exception:
    __version__ = "build"

from presalytics.client.api import Client
from presalytics.lib.plugins.base import PluginBase
from presalytics.lib.plugins.external import ApprovedExternalLinks, ApprovedExternalScripts
from presalytics.lib.plugins.jinja import JinjaPluginMakerMixin
from presalytics.lib.plugins.local import LocalStylesPlugin
from presalytics.lib.plugins.matplotlib import Mpld3Plugin
from presalytics.lib.plugins.ooxml import OoxmlTheme
from presalytics.lib.plugins.reveal import RevealConfigPlugin
from presalytics.lib.plugins.reveal_theme import RevealCustomTheme
from presalytics.lib.plugins.scss import ScssPlugin
from presalytics.lib.templates.base import (
    JinjaTemplateBuilder,
    BootstrapCustomTemplate
)
from presalytics.lib.widgets.matplotlib import MatplotlibFigure, MatplotlibResponsiveFigure
from presalytics.lib.widgets.d3 import (
    D3Widget
)
from presalytics.lib.widgets.chart import ChartWidget
from presalytics.lib.widgets.data_table import DataTableWidget
from presalytics.lib.widgets.url import UrlWidget
from presalytics.lib.widgets.markdown import MarkdownWidget
from presalytics.lib.widgets.ooxml import (
    OoxmlWidgetBase,
    OoxmlFileWidget, 
    OoxmlEndpointMap,
    ChartUpdaterWidget,
    TableUpdaterWidget
)
from presalytics.story.outline import StoryOutline
from presalytics.story.revealer import Revealer
from presalytics.story.components import WidgetBase, PageTemplateBase, Renderer, ThemeBase
from presalytics.lib.tools.ooxml_tools import (
    create_story_from_ooxml_file
)
from presalytics.lib.tools.story_tools import story_post_file_bytes
from presalytics.lib.tools.component_tools import (
    create_outline_from_page,
    create_outline_from_widget
)
from presalytics.lib.widgets.ooxml_editors import (
    OoxmlEditorWidget,
    XmlTransformBase,
    ChangeShapeColor,
    TextReplace,
    MultiXmlTransform
)



__all__ = [
    'CONFIG',
    'COMPONENTS',
    'PLUGINS',
    'Client',
    'StoryOutline',
    'Renderer',
    'Revealer',
    'MatplotlibFigure',
    'MatplotlibResponsiveFigure',
    'OoxmlFileWidget',
    'OoxmlEndpointMap',
    'OoxmlWidgetBase',
    'OoxmlEditorWidget',
    'D3Widget',
    'ChartWidget',
    'DataTableWidget',
    'UrlWidget',
    'ChartUpdaterWidget',
    'TableUpdaterWidget',
    'XmlTransformBase',
    'ChangeShapeColor',
    'TextReplace',
    'MultiXmlTransform',
    'ApprovedExternalLinks',
    'ApprovedExternalScripts',
    'LocalStylesPlugin',
    'OoxmlTheme',
    'RevealConfigPlugin',
    'RevealCustomTheme',
    'JinjaTemplateBuilder',
    'WidgetBase',
    'PageTemplateBase',
    'ScssPlugin',
    'create_story_from_ooxml_file',
    'story_post_file_bytes',
    'create_outline_from_page',
    'create_outline_from_widget'
]


import json
import logging
import pkg_resources
import sys
import os
import weakref
import dateutil
import dateutil.parser
import datetime
import six
import posixpath
import presalytics
import presalytics.lib.exceptions
import presalytics.lib.constants
import presalytics.story.outline
import typing
import mimetypes
import abc
from werkzeug.datastructures import FileStorage
if typing.TYPE_CHECKING:
    from presalytics.client.api import Client


logger = logging.getLogger(__name__)


class TokenUtil(object):
    def __init__(self, token=None, token_file=None, token_cache=False):
        self.token_cache = token_cache
        if token_file is None:
            self.token_file = presalytics.lib.constants.TOKEN_FILE
        else:
            self.token_file = token_file
        try:
            self._load_token_file()
        except Exception:
            pass
        if token is not None:
            try:
                self.process_token(token)    
                if self.token_cache:
                    self._put_token_file()
            except Exception:
                raise presalytics.lib.exceptions.MisConfiguredTokenException()
        else:
            if not hasattr(self, 'token'):
                self.token = {}

    def is_api_access_token_expired(self):
        try:
            if not self.token.get('access_token_expire_time', None):
                if self.token.get('access_token', None):
                    return False  # If expire time unknown, instruct client to call endpoint
                else:
                    return True  # If no token, instruct client to acquire token
            token_expiry = self.token['access_token_expire_time']
            if isinstance(token_expiry, datetime.datetime):
                expire_datetime = token_expiry.astimezone(datetime.timezone.utc)
            elif isinstance(token_expiry, str):
                expire_datetime = dateutil.parser.parse(self.token['access_token_expire_time']).astimezone(datetime.timezone.utc)
            else:
                logger.error("Token expire time not of type str (isoformat) or datetime.datetime")
                return True  # Get a new token on type error
            if expire_datetime < datetime.datetime.utcnow().astimezone(datetime.timezone.utc):
                return True
            else:
                return False
        except Exception as ex:
            logger.exception(ex)
            return True # Get a new token on unknown errors

    def _load_token_file(self):
        try:
            if self.token_cache:
                self.token = self.load_token_from_file(self.token_file)
        except Exception:
            logger.error("Unable to load token from cache.  If you do intend to cache tokens, use configuration CACHE_TOKENS=False")

    def _put_token_file(self):
        try:
            if self.token_cache:
                self.put_token_file(self.token, self.token_file)
        except Exception:
            logger.error("Failed to cache token.  Likely a write permissions error for the filesystem.")

    def process_token(self, token):
        self.token = {
            'access_token': token['access_token']
        }
        if not token.get('access_token_expire_time', None):
            if token.get('expires_in', None):
                access_token_expire_time = datetime.datetime.utcnow().astimezone(datetime.timezone.utc) + datetime.timedelta(seconds=token['expires_in'])
            else:
                pass    # TODO: add logic to introspect token for expire time 
        else:
            access_token_expire_time = token["access_token_expire_time"]
        if access_token_expire_time:
            self.token.update({
                "access_token_expire_time": access_token_expire_time
            })
        if token.get('refresh_token', None):
            self.token.update({
                "refresh_token": token["refresh_token"]
            })
        return self.token

    @staticmethod
    def put_token_file(token, token_filepath):
        with open(token_filepath, mode='w+') as newtoken:
            json.dump(token, newtoken, cls=presalytics.story.outline.OutlineEncoder)

    @staticmethod
    def load_token_from_file(token_filepath):
        with open(token_filepath, 'r') as token_file:
            token = json.load(token_file)
        return token


class AuthenticationMixIn(abc.ABC):
    def __init__(self, parent: 'Client', ignore_api_exceptions=False, **kwargs):
        self._ignore_api_exceptions = ignore_api_exceptions
        self.parent = weakref.ref(parent)
        super(AuthenticationMixIn, self).__init__()
        self.update_configuration()

    @property
    @abc.abstractmethod
    def api_name(self):
        return NotImplemented

    def call_api(
            self, resource_path, method,
            path_params=None, query_params=None, header_params=None,
            body=None, post_params=None, files=None,
            response_type=None, auth_settings=None, async_req=None,
            _return_http_data_only=None, collection_formats=None,
            _preload_content=None, _request_timeout=None, _host=None):
        """
        Overriding call_api to force token check, refresh on each api call,
        rather than at class initialized (good for ipython notebooks)
        """
        if self.parent() is None:
            message = """
            Missing reference to Client class.  Client was like garbage collected by the intepreter.\n  
            Please initialize the Client class on its own line to avoid this error.  For example:\n\n
            client = presalytics.Client()\n
            story = client.story.story_id_get(story_id)
            """
            raise presalytics.lib.exceptions.InvalidConfigurationError(message=message)
        auth_header = self.parent().get_auth_header()
        if header_params is not None:
            header_params.update(auth_header)
        else:
            header_params = auth_header
        if not header_params.get("X-Request_Id"):
            req_id = self.parent().get_request_id_header()
            header_params.update(req_id)
        request_id = header_params.get("X-Request-Id")
        try:

            endpoint = self.configuration.host + resource_path
            logger.info("Sending {0} message to {1}. Request Id: {2}".format(method, endpoint, request_id))
            call_args = (
                resource_path, 
                method, 
                path_params,
                query_params, 
                header_params, 
                body, 
                post_params, 
                files, 
                response_type,
                auth_settings, 
                async_req, 
                _return_http_data_only, 
                collection_formats, 
                _preload_content,
                _request_timeout, 
                _host
            )
            response = super(AuthenticationMixIn, self).call_api(*call_args)
            logger.info("{0} response received from {1}".format(method, endpoint))
            return response
        except Exception as e:
            if type(e).__name__ == "ApiException":
                try:
                    d = json.loads(e.body)
                    addendum = " Host: {0}; Path: {1}; Method: {2}; Request Id: {3}".format(self.configuration.host, resource_path, method, request_id)
                    try:
                        d["detail"] = d["detail"] + addendum
                    except TypeError:
                        d = e.body + addendum
                    e.body = json.dumps(d)
                    logging.error(d)
                    if d["status"] == 401:
                        logging.debug("Refeshing token from unauathorized call and retrying")
                        self.parent().refresh_token()
                        return super(AuthenticationMixIn, self).call_api(*call_args)
                except Exception:
                    pass
                if self._ignore_api_exceptions:
                    return e.body, e.status, e.headers
                else:
                    raise presalytics.lib.exceptions.ApiException(default_exception=e)
            else:
                t, v, tb = sys.exc_info()
                six.reraise(t, v, tb)

    def update_configuration(self):
        """
        updates the configuration from the base api client to parameters contained in the config file.
        base api client must be initialized prior to calling this method (i.e., self.configuration cannot equal None),
        so that configuration can automatically inherit default values.
        Mostly used for debugging purposes, but self-hosted API endpoints may require injection of these parameters
        """
        if self.configuration is None:
            raise presalytics.lib.exceptions.MissingConfigException("Base API not yet configured, please reconstruct API initialization")
        self.user_agent = AuthenticationMixIn._get_user_agent
        if presalytics.CONFIG.get("HOSTS", None):
            self.set_host(presalytics.CONFIG.get('HOSTS'))

    @staticmethod
    def get_user_agent():
        try:
            VER = pkg_resources.require("presalytics")[0].version
        except Exception:
            VER = 'build'
        return "presalytics-python-client/{0}".format(VER)

    _get_user_agent = get_user_agent.__func__()  #type: ignore

    def set_host(self, hosts_dict):
        for parent_cls in self.__class__.__bases__:
            if parent_cls.__name__ == 'ApiClient':
                for k, v in hosts_dict.items():
                    if k.lower() in parent_cls.__module__:
                        host_key = k
                        break
        try:
            self.configuration.host = hosts_dict[host_key]
        except (KeyError, UnboundLocalError):
            pass

    def _ApiClient__deserialize_datetime(self, string):
        """Overwrites generated function to include UTC timezone on datetime object
        needs to account for name-mangling in parent class
        The string should be in iso8601 datetime format.

        :param string: str.
        :return: datetime.
        """
        return super()._ApiClient__deserialize_datetime(string).astimezone(datetime.timezone.utc)

    @property
    def external_root_url(self):
        service_key = self.api_name.replace("-", "_").upper()
        if presalytics.CONFIG.get("BROWSER_API_HOST", dict()).get(service_key, None):
            service_host = presalytics.CONFIG["BROWSER_API_HOST"][service_key]
            target = service_host + "/" + self.api_name
        else:
            target = self.configuration.host
        return target


    def files_parameters(self, files=None):
        """Builds form parameters.

        This override method expands the capabilites of codegen filehandler to
        accept a `werkzeug.datastructures.FileStorage` object.

        :param files: File parameters. Either a string file path or a `werkzeug.datastructures.FileStorage` object
        :return: Form parameters with files.
        """
        params = []

        if files:
            for k, v in six.iteritems(files):
                if not v:
                    continue
                if type(v) is str or type(v) is list:
                    file_names = v if type(v) is list else [v]
                    for n in file_names:
                        with open(n, 'rb') as f:
                            filename = os.path.basename(f.name)
                            filedata = f.read()

                else:
                    if type(v) is FileStorage:
                        filename = v.filename
                        v.stream.seek(0)
                        filedata = v.stream.read()
                    else:
                        raise AttributeError("Invalid File Object")
                mimetype = (mimetypes.guess_type(filename)[0] or 'application/octet-stream')
                params.append(tuple([k, tuple([filename, filedata, mimetype])]))

        return params


# coding: utf-8

"""
    Story

    This API is the main entry point for creating, editing and publishing analytics throught the Presalytics API  # noqa: E501

    The version of the OpenAPI document: 0.3.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class ViewAllOf(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'active_msecs': 'int',
        'additional': 'str',
        'end_time': 'datetime',
        'page_number': 'int',
        'session_id': 'str',
        'start_time': 'datetime',
        'total_msecs': 'int'
    }

    attribute_map = {
        'active_msecs': 'active_msecs',
        'additional': 'additional',
        'end_time': 'end_time',
        'page_number': 'page_number',
        'session_id': 'session_id',
        'start_time': 'start_time',
        'total_msecs': 'total_msecs'
    }

    def __init__(self, active_msecs=None, additional=None, end_time=None, page_number=None, session_id=None, start_time=None, total_msecs=None):  # noqa: E501
        """ViewAllOf - a model defined in OpenAPI"""  # noqa: E501

        self._active_msecs = None
        self._additional = None
        self._end_time = None
        self._page_number = None
        self._session_id = None
        self._start_time = None
        self._total_msecs = None
        self.discriminator = None

        if active_msecs is not None:
            self.active_msecs = active_msecs
        if additional is not None:
            self.additional = additional
        if end_time is not None:
            self.end_time = end_time
        if page_number is not None:
            self.page_number = page_number
        if session_id is not None:
            self.session_id = session_id
        if start_time is not None:
            self.start_time = start_time
        if total_msecs is not None:
            self.total_msecs = total_msecs

    @property
    def active_msecs(self):
        """Gets the active_msecs of this ViewAllOf.  # noqa: E501


        :return: The active_msecs of this ViewAllOf.  # noqa: E501
        :rtype: int
        """
        return self._active_msecs

    @active_msecs.setter
    def active_msecs(self, active_msecs):
        """Sets the active_msecs of this ViewAllOf.


        :param active_msecs: The active_msecs of this ViewAllOf.  # noqa: E501
        :type: int
        """

        self._active_msecs = active_msecs

    @property
    def additional(self):
        """Gets the additional of this ViewAllOf.  # noqa: E501


        :return: The additional of this ViewAllOf.  # noqa: E501
        :rtype: str
        """
        return self._additional

    @additional.setter
    def additional(self, additional):
        """Sets the additional of this ViewAllOf.


        :param additional: The additional of this ViewAllOf.  # noqa: E501
        :type: str
        """

        self._additional = additional

    @property
    def end_time(self):
        """Gets the end_time of this ViewAllOf.  # noqa: E501


        :return: The end_time of this ViewAllOf.  # noqa: E501
        :rtype: datetime
        """
        return self._end_time

    @end_time.setter
    def end_time(self, end_time):
        """Sets the end_time of this ViewAllOf.


        :param end_time: The end_time of this ViewAllOf.  # noqa: E501
        :type: datetime
        """

        self._end_time = end_time

    @property
    def page_number(self):
        """Gets the page_number of this ViewAllOf.  # noqa: E501


        :return: The page_number of this ViewAllOf.  # noqa: E501
        :rtype: int
        """
        return self._page_number

    @page_number.setter
    def page_number(self, page_number):
        """Sets the page_number of this ViewAllOf.


        :param page_number: The page_number of this ViewAllOf.  # noqa: E501
        :type: int
        """

        self._page_number = page_number

    @property
    def session_id(self):
        """Gets the session_id of this ViewAllOf.  # noqa: E501


        :return: The session_id of this ViewAllOf.  # noqa: E501
        :rtype: str
        """
        return self._session_id

    @session_id.setter
    def session_id(self, session_id):
        """Sets the session_id of this ViewAllOf.


        :param session_id: The session_id of this ViewAllOf.  # noqa: E501
        :type: str
        """

        self._session_id = session_id

    @property
    def start_time(self):
        """Gets the start_time of this ViewAllOf.  # noqa: E501


        :return: The start_time of this ViewAllOf.  # noqa: E501
        :rtype: datetime
        """
        return self._start_time

    @start_time.setter
    def start_time(self, start_time):
        """Sets the start_time of this ViewAllOf.


        :param start_time: The start_time of this ViewAllOf.  # noqa: E501
        :type: datetime
        """

        self._start_time = start_time

    @property
    def total_msecs(self):
        """Gets the total_msecs of this ViewAllOf.  # noqa: E501


        :return: The total_msecs of this ViewAllOf.  # noqa: E501
        :rtype: int
        """
        return self._total_msecs

    @total_msecs.setter
    def total_msecs(self, total_msecs):
        """Sets the total_msecs of this ViewAllOf.


        :param total_msecs: The total_msecs of this ViewAllOf.  # noqa: E501
        :type: int
        """

        self._total_msecs = total_msecs

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ViewAllOf):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class SharedEffectsDetails(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'name': 'str',
        'effect_attributes': 'list[SharedEffectAttributesDetails]',
        'shape_id': 'str',
        'parent_shape': 'SlideShapesDetails',
        'connector_id': 'str',
        'parent_connector': 'SlideConnectorDetails',
        'effect_map_id': 'str',
        'effect_map': 'ThemeEffectMapDetails',
        'id': 'str',
        'date_created': 'datetime',
        'user_created': 'str',
        'date_modified': 'datetime',
        'user_modified': 'str'
    }

    attribute_map = {
        'name': 'name',
        'effect_attributes': 'effectAttributes',
        'shape_id': 'shapeId',
        'parent_shape': 'parentShape',
        'connector_id': 'connectorId',
        'parent_connector': 'parentConnector',
        'effect_map_id': 'effectMapId',
        'effect_map': 'effectMap',
        'id': 'id',
        'date_created': 'dateCreated',
        'user_created': 'userCreated',
        'date_modified': 'dateModified',
        'user_modified': 'userModified'
    }

    def __init__(self, name=None, effect_attributes=None, shape_id=None, parent_shape=None, connector_id=None, parent_connector=None, effect_map_id=None, effect_map=None, id=None, date_created=None, user_created=None, date_modified=None, user_modified=None):  # noqa: E501
        """SharedEffectsDetails - a model defined in OpenAPI"""  # noqa: E501

        self._name = None
        self._effect_attributes = None
        self._shape_id = None
        self._parent_shape = None
        self._connector_id = None
        self._parent_connector = None
        self._effect_map_id = None
        self._effect_map = None
        self._id = None
        self._date_created = None
        self._user_created = None
        self._date_modified = None
        self._user_modified = None
        self.discriminator = None

        self.name = name
        self.effect_attributes = effect_attributes
        self.shape_id = shape_id
        if parent_shape is not None:
            self.parent_shape = parent_shape
        self.connector_id = connector_id
        if parent_connector is not None:
            self.parent_connector = parent_connector
        self.effect_map_id = effect_map_id
        if effect_map is not None:
            self.effect_map = effect_map
        if id is not None:
            self.id = id
        if date_created is not None:
            self.date_created = date_created
        if user_created is not None:
            self.user_created = user_created
        if date_modified is not None:
            self.date_modified = date_modified
        if user_modified is not None:
            self.user_modified = user_modified

    @property
    def name(self):
        """Gets the name of this SharedEffectsDetails.  # noqa: E501


        :return: The name of this SharedEffectsDetails.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this SharedEffectsDetails.


        :param name: The name of this SharedEffectsDetails.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def effect_attributes(self):
        """Gets the effect_attributes of this SharedEffectsDetails.  # noqa: E501


        :return: The effect_attributes of this SharedEffectsDetails.  # noqa: E501
        :rtype: list[SharedEffectAttributesDetails]
        """
        return self._effect_attributes

    @effect_attributes.setter
    def effect_attributes(self, effect_attributes):
        """Sets the effect_attributes of this SharedEffectsDetails.


        :param effect_attributes: The effect_attributes of this SharedEffectsDetails.  # noqa: E501
        :type: list[SharedEffectAttributesDetails]
        """

        self._effect_attributes = effect_attributes

    @property
    def shape_id(self):
        """Gets the shape_id of this SharedEffectsDetails.  # noqa: E501


        :return: The shape_id of this SharedEffectsDetails.  # noqa: E501
        :rtype: str
        """
        return self._shape_id

    @shape_id.setter
    def shape_id(self, shape_id):
        """Sets the shape_id of this SharedEffectsDetails.


        :param shape_id: The shape_id of this SharedEffectsDetails.  # noqa: E501
        :type: str
        """

        self._shape_id = shape_id

    @property
    def parent_shape(self):
        """Gets the parent_shape of this SharedEffectsDetails.  # noqa: E501


        :return: The parent_shape of this SharedEffectsDetails.  # noqa: E501
        :rtype: SlideShapesDetails
        """
        return self._parent_shape

    @parent_shape.setter
    def parent_shape(self, parent_shape):
        """Sets the parent_shape of this SharedEffectsDetails.


        :param parent_shape: The parent_shape of this SharedEffectsDetails.  # noqa: E501
        :type: SlideShapesDetails
        """

        self._parent_shape = parent_shape

    @property
    def connector_id(self):
        """Gets the connector_id of this SharedEffectsDetails.  # noqa: E501


        :return: The connector_id of this SharedEffectsDetails.  # noqa: E501
        :rtype: str
        """
        return self._connector_id

    @connector_id.setter
    def connector_id(self, connector_id):
        """Sets the connector_id of this SharedEffectsDetails.


        :param connector_id: The connector_id of this SharedEffectsDetails.  # noqa: E501
        :type: str
        """

        self._connector_id = connector_id

    @property
    def parent_connector(self):
        """Gets the parent_connector of this SharedEffectsDetails.  # noqa: E501


        :return: The parent_connector of this SharedEffectsDetails.  # noqa: E501
        :rtype: SlideConnectorDetails
        """
        return self._parent_connector

    @parent_connector.setter
    def parent_connector(self, parent_connector):
        """Sets the parent_connector of this SharedEffectsDetails.


        :param parent_connector: The parent_connector of this SharedEffectsDetails.  # noqa: E501
        :type: SlideConnectorDetails
        """

        self._parent_connector = parent_connector

    @property
    def effect_map_id(self):
        """Gets the effect_map_id of this SharedEffectsDetails.  # noqa: E501


        :return: The effect_map_id of this SharedEffectsDetails.  # noqa: E501
        :rtype: str
        """
        return self._effect_map_id

    @effect_map_id.setter
    def effect_map_id(self, effect_map_id):
        """Sets the effect_map_id of this SharedEffectsDetails.


        :param effect_map_id: The effect_map_id of this SharedEffectsDetails.  # noqa: E501
        :type: str
        """

        self._effect_map_id = effect_map_id

    @property
    def effect_map(self):
        """Gets the effect_map of this SharedEffectsDetails.  # noqa: E501


        :return: The effect_map of this SharedEffectsDetails.  # noqa: E501
        :rtype: ThemeEffectMapDetails
        """
        return self._effect_map

    @effect_map.setter
    def effect_map(self, effect_map):
        """Sets the effect_map of this SharedEffectsDetails.


        :param effect_map: The effect_map of this SharedEffectsDetails.  # noqa: E501
        :type: ThemeEffectMapDetails
        """

        self._effect_map = effect_map

    @property
    def id(self):
        """Gets the id of this SharedEffectsDetails.  # noqa: E501


        :return: The id of this SharedEffectsDetails.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SharedEffectsDetails.


        :param id: The id of this SharedEffectsDetails.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def date_created(self):
        """Gets the date_created of this SharedEffectsDetails.  # noqa: E501


        :return: The date_created of this SharedEffectsDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_created

    @date_created.setter
    def date_created(self, date_created):
        """Sets the date_created of this SharedEffectsDetails.


        :param date_created: The date_created of this SharedEffectsDetails.  # noqa: E501
        :type: datetime
        """

        self._date_created = date_created

    @property
    def user_created(self):
        """Gets the user_created of this SharedEffectsDetails.  # noqa: E501


        :return: The user_created of this SharedEffectsDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_created

    @user_created.setter
    def user_created(self, user_created):
        """Sets the user_created of this SharedEffectsDetails.


        :param user_created: The user_created of this SharedEffectsDetails.  # noqa: E501
        :type: str
        """

        self._user_created = user_created

    @property
    def date_modified(self):
        """Gets the date_modified of this SharedEffectsDetails.  # noqa: E501


        :return: The date_modified of this SharedEffectsDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_modified

    @date_modified.setter
    def date_modified(self, date_modified):
        """Sets the date_modified of this SharedEffectsDetails.


        :param date_modified: The date_modified of this SharedEffectsDetails.  # noqa: E501
        :type: datetime
        """

        self._date_modified = date_modified

    @property
    def user_modified(self):
        """Gets the user_modified of this SharedEffectsDetails.  # noqa: E501


        :return: The user_modified of this SharedEffectsDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_modified

    @user_modified.setter
    def user_modified(self, user_modified):
        """Sets the user_modified of this SharedEffectsDetails.


        :param user_modified: The user_modified of this SharedEffectsDetails.  # noqa: E501
        :type: str
        """

        self._user_modified = user_modified

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SharedEffectsDetails):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import jinja2
import presalytics.lib.exceptions
import typing


class JinjaPluginMakerMixin(object):
    """
    Mixin that helps plugins use Jinja2 python library for dynamically rendering 
    plugins based on the config dictionary.  Values from the `config` are pass 
    to the Jinja2 templating engine rendered into the html fragment via a 
    double-handlered placeholder (e.g., `{{ config_dict_key }}`)

    For more information on how to design a Jinja2 template, visit 
    https://jinja.palletsprojects.com

    Attributes
    ----------
    template : str
        the template property on a class that takes this mixin must 
        return a string with double-handlebarred placeholders

    Configuration Values
    ----------
    jinja_options : dict
        Options to be passed to the Jinja2 environment.  See a list of options on the 
        [Jinja2 API Page](https://jinja.palletsprojects.com/en/2.11.x/api/#jinja2.Environment)

    **kwargs : dict
        Other keyword arguments will be passed as variables to the Jinja2 rendering engine to that
        fill placeholders in the template
    """
    template: typing.Optional[str]
    template = None

    def __init__(self, *args, **kwargs):
        super(JinjaPluginMakerMixin, self).__init__(*args, *kwargs)

    def render(self, config: typing.Dict):
        """
        Loads the Configuration Value into the template and returns the filled out html fragment
        """
        options = {
            "loader": jinja2.BaseLoader()
        }
        if config.get("jinja_options", None):
            options.update(config.pop("jinja_options"))
        template = jinja2.Environment(**options).from_string(self.get_template())
        return template.render(config)

    def get_template(self) -> str:
        if not self.template:
            raise presalytics.lib.exceptions.ValidationError("Plugins the use the JinjaPluginMakerMixin must have a defiined template attribute.")
        else:
            return self.template



import sass
import os
import posixpath
import string
import typing
import presalytics.lib.plugins.reveal_theme
import presalytics.story.util


class OoxmlTheme(presalytics.lib.plugins.reveal_theme.RevealCustomTheme):
    """
    This class takes theme data from an Presalytics API Ooxml Automation service Theme object
    and feeds those values into the `presalytics.lib.plugins.reveal_theme.RevealCustomTheme`
    """
    __plugin_name__ = 'ooxml-theme'

    def to_style(self, config, **kwargs):
        base_path = os.path.join(os.path.dirname(__file__), 'scss')
        if os.name == "nt":
            import_path = posixpath.join(*base_path.split('\\'))
        else:
            import_path = base_path
        scss_file = os.path.join(base_path, 'overrides.tmpl')
        config.update({'path': import_path})
        with open(scss_file, 'r') as file:
            scss_template_string = file.read()
        config = self.config_to_camelCase(config)
        scss_string = scss_template_string.format(**config)
        new_css = sass.compile(string=scss_string)
        style_string = "<style>\n{0}\n</style>".format(new_css)
        font_names = [config["headingFont"], config["bodyFont"]]
        links = self.get_fonts(font_names)
        return links + style_string
    
    def config_to_camelCase(self, config: typing.Dict) -> typing.Dict:
        new_dict = dict()
        for key, val in config.items():
            new_dict.update({
                presalytics.story.util.to_camel_case(key): val
            })
        return new_dict




# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class ChartChartDataDTO(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'chart_id': 'str',
        'series_names': 'list[str]',
        'category_names': 'list[str]',
        'data_points': 'list[list[float]]'
    }

    attribute_map = {
        'chart_id': 'chartId',
        'series_names': 'seriesNames',
        'category_names': 'categoryNames',
        'data_points': 'dataPoints'
    }

    def __init__(self, chart_id=None, series_names=None, category_names=None, data_points=None):  # noqa: E501
        """ChartChartDataDTO - a model defined in OpenAPI"""  # noqa: E501

        self._chart_id = None
        self._series_names = None
        self._category_names = None
        self._data_points = None
        self.discriminator = None

        if chart_id is not None:
            self.chart_id = chart_id
        self.series_names = series_names
        self.category_names = category_names
        self.data_points = data_points

    @property
    def chart_id(self):
        """Gets the chart_id of this ChartChartDataDTO.  # noqa: E501


        :return: The chart_id of this ChartChartDataDTO.  # noqa: E501
        :rtype: str
        """
        return self._chart_id

    @chart_id.setter
    def chart_id(self, chart_id):
        """Sets the chart_id of this ChartChartDataDTO.


        :param chart_id: The chart_id of this ChartChartDataDTO.  # noqa: E501
        :type: str
        """

        self._chart_id = chart_id

    @property
    def series_names(self):
        """Gets the series_names of this ChartChartDataDTO.  # noqa: E501


        :return: The series_names of this ChartChartDataDTO.  # noqa: E501
        :rtype: list[str]
        """
        return self._series_names

    @series_names.setter
    def series_names(self, series_names):
        """Sets the series_names of this ChartChartDataDTO.


        :param series_names: The series_names of this ChartChartDataDTO.  # noqa: E501
        :type: list[str]
        """

        self._series_names = series_names

    @property
    def category_names(self):
        """Gets the category_names of this ChartChartDataDTO.  # noqa: E501


        :return: The category_names of this ChartChartDataDTO.  # noqa: E501
        :rtype: list[str]
        """
        return self._category_names

    @category_names.setter
    def category_names(self, category_names):
        """Sets the category_names of this ChartChartDataDTO.


        :param category_names: The category_names of this ChartChartDataDTO.  # noqa: E501
        :type: list[str]
        """

        self._category_names = category_names

    @property
    def data_points(self):
        """Gets the data_points of this ChartChartDataDTO.  # noqa: E501


        :return: The data_points of this ChartChartDataDTO.  # noqa: E501
        :rtype: list[list[float]]
        """
        return self._data_points

    @data_points.setter
    def data_points(self, data_points):
        """Sets the data_points of this ChartChartDataDTO.


        :param data_points: The data_points of this ChartChartDataDTO.  # noqa: E501
        :type: list[list[float]]
        """

        self._data_points = data_points

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ChartChartDataDTO):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import pkgutil
import importlib
import traceback
import sys
import os
import inspect
import logging
import typing
import abc
import re
import types
import ast
import presalytics.lib.exceptions


logger = logging.getLogger('presalytics.lib.registry')


class RegistryBase(abc.ABC):
    """
    In-memory container for python objects that return attributes from both the
    `get_type` and `get_name` abstract functions. 

    The `discover` method searches the current working directory, directories at
    in the `autodiscover_paths` attribute, and packages in the current python environment
    prefixed with "presalytics" (i.e., extensions).
    """
    registry: typing.Dict[str, typing.Type]
    deferred_modules: typing.List[typing.Dict[str, typing.Any]]
    show_errors = False

    def __init__(self, 
                 show_errors=False, 
                 autodiscover_paths=[], 
                 reserved_names: typing.List[str] = None, 
                 ignore_paths: typing.List[str] = None,
                 **kwargs):
        RegistryBase.show_errors = show_errors
        self.error_class = presalytics.lib.exceptions.RegistryError
        self.autodiscover_paths = autodiscover_paths
        self.ignore_paths = ignore_paths if ignore_paths else []
        self.registry = {}
        self.reserved_names = ["config.py", "setup.py"]
        self.deferred_modules = []  # modules to load at at runtime, if theres a ciruclat dependency at import-time
        try:
            if reserved_names:
                self.reserved_names.extend(reserved_names)
        except Exception:
            pass
        remove_paths = []
        for path in self.ignore_paths:
            for search_path in self.autodiscover_paths:
                if path in search_path:
                    remove_paths.append(search_path)
        for remove_path in remove_paths:
            self.autodiscover_paths.remove(remove_path)
        self.discover()
        self.key_regex = re.compile(r'(.*)\.(.*)')

    def raise_error(self, message):
        raise self.error_class(self, message)

    @abc.abstractmethod
    def get_type(self, klass):
        raise NotImplementedError

    @abc.abstractmethod
    def get_name(self, klass):
        raise NotImplementedError

    @staticmethod
    def onerror(name):
        if RegistryBase.show_errors:
            print("Error importing module %s" % name)
            t, value, tb = sys.exc_info()
            traceback.print_tb(tb)

    def get(self, key):
        self.load_deferred_modules()
        return self.registry.get(key, None)

    def get_classes_from_module(self, module):
        self.get_classes(module)
        if getattr(module, "__path__", None):
            for loader, name, is_pkg in pkgutil.walk_packages(module.__path__, onerror=RegistryBase.onerror):
                full_name = module.__name__ + '.' + name
                try:
                    if not self.module_is_in_stackframe(full_name):
                        sub_module = importlib.import_module(full_name)
                        if sub_module.__name__.startswith('presalytics'):
                            if is_pkg:
                                self.get_classes_from_module(sub_module)
                            else:
                                self.get_classes(sub_module)
                except Exception:
                    pass

    def get_registry_key(self, klass):
        key = None
        klass_type = self.get_type(klass)
        if klass_type:
            klass_name = self.get_name(klass)
            if klass_name:
                key = "{0}.{1}".format(klass_type, klass_name)
        return key

    def load_class(self, klass):
        if inspect.isclass(klass):
            try:
                key = self.get_registry_key(klass)
                if key:
                    if key not in self.registry.keys():
                        self.registry[key] = klass
            except Exception as ex:
                if self.show_errors:
                    logger.exception(ex)
                    klass_type = self.get_type(klass)
                    message = "Unable to register class {0} with type {1}".format(klass.__name__, klass_type)
                    logger.error(message)
        else:
            try:
                if self.show_errors:
                    message = "{0} is not a class".format(klass.__class__.__name__)
                    logger.error(message)
            except Exception:
                pass

    def get_classes(self, module):
        for key, val in module.__dict__.items():
            if inspect.isclass(val) or isinstance(val, abc.ABC):
                self.load_class(val)


    def load_deferred_modules(self):
        if len(self.deferred_modules) > 0:
            new_deferred = []
            for mod in self.deferred_modules:
                try:
                    name = mod.get("name") 
                    if name in sys.modules: 
                        module = sys.modules[name] 
                    else: 
                        spec = mod.get("spec")
                        module = mod.get("module")
                        sys.modules[name] = module
                        spec.loader.exec_module(module) 
                    self.get_classes(module)
                except Exception as ex:
                    logger.exception(ex)
                    message = "Failure to execute deferred load on module '{}'.  Please check exception message and review for errors.".format(mod.get("name", None))
                    logger.error(message)
                    new_deferred.append(mod)
            self.deferred_modules = new_deferred # removes modules successfully loaded from the list
                 
                
    def discover(self):
        current_path = os.getcwd()
        if current_path not in self.autodiscover_paths:
            self.autodiscover_paths.append(current_path)
        for path in self.autodiscover_paths:
            for name in os.listdir(path):
                if name.endswith(".py") and name not in self.reserved_names:
                    module_name = name.replace(".py", "")
                    try:
                        mod_path = os.path.join(path, name)
                        mod_spec = importlib.util.spec_from_file_location(module_name, mod_path)
                        if not self.module_is_in_stackframe(module_name):
                            """
                            Defer workspace-level imports so these modules only load once
                            This avoids memory errors in computationally-intensive packages (in some environments)
                            and packages built using Intel's MKL (e.g., scipy, numpy, sklearn), which
                            raises ValueErrors when modules are re-executed under the same interpreter w/o a ipython kernel.
                            TODO: Add note to docs saying not to put components and plugins in the same file
                            """
                            mod = importlib.util.module_from_spec(mod_spec)
                            self.deferred_modules.append({
                                "name": module_name,
                                "module": mod,
                                "spec": mod_spec
                            })
                    except (AttributeError, ImportError) as circ:
                        # Checks for targets of circular imports, and defer those imports to runtime
                        message = "Likely circular import in module '{}'. Deferring import to run-time.".format(mod.__name__)
                        logger.info(message)
                        self.deferred_modules.append({
                            "name": mod.__name__,
                            "module": mod,
                            "spec": mod_spec
                        })
                    except Exception as ex:
                        if self.show_errors:
                            logger.exception(ex)
                            message = "Could not load classes from file {0}".format(name)
                            logger.error(message)
        for finder, name, ispkg in pkgutil.iter_modules():
            if name.startswith('presalytics'):
                mod = importlib.import_module(name)
                self.get_classes_from_module(mod)

    def module_is_in_stackframe(self, module_name, frame=None) -> bool:
        """Prevent cycles by skipping modules already loaded in the stack frame
        """
        in_stack = False
        try:
            if not frame:
                frame = inspect.currentframe()
            frame_module = frame.f_globals['__name__']
            if frame_module == '__main__':
                try:
                    frame_module = inspect.getmodule(frame).__spec__.name #type: ignore
                except AttributeError:
                    # vscode and spyder's parent controllers load __main__ without a module spec
                    if not getattr(inspect.getmodule(frame), "__spec__", None):
                        try:
                            fname = inspect.getmodule(frame).__dict__['__file__']
                            frame_module = os.path.basename(fname).replace(".py", "")
                            if module_name == frame_module:
                                return True
                            else: 
                                return False
                        except AttributeError:
                            return in_stack
                        except Exception:
                            return True
                    else:
                        return True # Don't load this module for unknown errors
                except Exception:
                    return True
            if module_name == frame_module:
                in_stack = True
            else:
                if frame.f_back:
                    in_stack = self.module_is_in_stackframe(module_name, frame=frame.f_back)
        except Exception:
            # if something wonky happens, don't load this module
            # if loaded in IronPython or Jython, this will probably fire (untested)
            in_stack = True
        finally:
            del frame
        return in_stack


    def register(self, klass):
        self.load_class(klass)
    
    def unregister(self, klass):
        key = self.get_registry_key(klass)
        if key:
            self.registry.pop(key)

    def find_class(self, string_with_key_or_name) -> typing.List[str]:
        is_key = self.key_regex.match(string_with_key_or_name)
        if is_key:
            return [self.get(string_with_key_or_name)]
        else:
            self.load_deferred_modules()
            return [x for x in self.registry.keys() if string_with_key_or_name in x]
            
            
            




# coding: utf-8

"""
    Story

    This API is the main entry point for creating, editing and publishing analytics throught the Presalytics API  # noqa: E501

    The version of the OpenAPI document: 0.3.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class FileUpload(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'content_length': 'int',
        'file': 'str',
        'file_name': 'str',
        'mimetype': 'str'
    }

    attribute_map = {
        'content_length': 'content_length',
        'file': 'file',
        'file_name': 'file_name',
        'mimetype': 'mimetype'
    }

    def __init__(self, content_length=None, file=None, file_name=None, mimetype=None):  # noqa: E501
        """FileUpload - a model defined in OpenAPI"""  # noqa: E501

        self._content_length = None
        self._file = None
        self._file_name = None
        self._mimetype = None
        self.discriminator = None

        if content_length is not None:
            self.content_length = content_length
        if file is not None:
            self.file = file
        if file_name is not None:
            self.file_name = file_name
        if mimetype is not None:
            self.mimetype = mimetype

    @property
    def content_length(self):
        """Gets the content_length of this FileUpload.  # noqa: E501


        :return: The content_length of this FileUpload.  # noqa: E501
        :rtype: int
        """
        return self._content_length

    @content_length.setter
    def content_length(self, content_length):
        """Sets the content_length of this FileUpload.


        :param content_length: The content_length of this FileUpload.  # noqa: E501
        :type: int
        """

        self._content_length = content_length

    @property
    def file(self):
        """Gets the file of this FileUpload.  # noqa: E501


        :return: The file of this FileUpload.  # noqa: E501
        :rtype: str
        """
        return self._file

    @file.setter
    def file(self, file):
        """Sets the file of this FileUpload.


        :param file: The file of this FileUpload.  # noqa: E501
        :type: str
        """

        self._file = file

    @property
    def file_name(self):
        """Gets the file_name of this FileUpload.  # noqa: E501


        :return: The file_name of this FileUpload.  # noqa: E501
        :rtype: str
        """
        return self._file_name

    @file_name.setter
    def file_name(self, file_name):
        """Sets the file_name of this FileUpload.


        :param file_name: The file_name of this FileUpload.  # noqa: E501
        :type: str
        """

        self._file_name = file_name

    @property
    def mimetype(self):
        """Gets the mimetype of this FileUpload.  # noqa: E501


        :return: The mimetype of this FileUpload.  # noqa: E501
        :rtype: str
        """
        return self._mimetype

    @mimetype.setter
    def mimetype(self, mimetype):
        """Sets the mimetype of this FileUpload.


        :param mimetype: The mimetype of this FileUpload.  # noqa: E501
        :type: str
        """

        self._mimetype = mimetype

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, FileUpload):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class SlideShapesDetails(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'height': 'int',
        'width': 'int',
        'x_offset': 'int',
        'y_offset': 'int',
        'text_container': 'SharedTextContainerDetails',
        'group_elements_id': 'str',
        'group_element': 'SlideGroupElementsDetails',
        'ooxml_id': 'int',
        'svg_blob_url': 'str',
        'preset_type_id': 'str',
        'free_form_path_xml': 'str',
        'fill_map': 'SharedFillMapDetails',
        'is_theme_fill': 'bool',
        'effect': 'SharedEffectsDetails',
        'is_theme_effect': 'bool',
        'line': 'SharedLinesDetails',
        'is_theme_line': 'bool',
        'flip_horizontal': 'bool',
        'flip_vertical': 'bool',
        'rotation': 'int',
        'hidden': 'bool',
        'base_element_blob_url': 'str',
        'changed_base_element_blob_url': 'str',
        'package_uri': 'str',
        'name': 'str',
        'id': 'str',
        'date_created': 'datetime',
        'user_created': 'str',
        'date_modified': 'datetime',
        'user_modified': 'str'
    }

    attribute_map = {
        'height': 'height',
        'width': 'width',
        'x_offset': 'xOffset',
        'y_offset': 'yOffset',
        'text_container': 'textContainer',
        'group_elements_id': 'groupElementsId',
        'group_element': 'groupElement',
        'ooxml_id': 'ooxmlId',
        'svg_blob_url': 'svgBlobUrl',
        'preset_type_id': 'presetTypeId',
        'free_form_path_xml': 'freeFormPathXml',
        'fill_map': 'fillMap',
        'is_theme_fill': 'isThemeFill',
        'effect': 'effect',
        'is_theme_effect': 'isThemeEffect',
        'line': 'line',
        'is_theme_line': 'isThemeLine',
        'flip_horizontal': 'flipHorizontal',
        'flip_vertical': 'flipVertical',
        'rotation': 'rotation',
        'hidden': 'hidden',
        'base_element_blob_url': 'baseElementBlobUrl',
        'changed_base_element_blob_url': 'changedBaseElementBlobUrl',
        'package_uri': 'packageUri',
        'name': 'name',
        'id': 'id',
        'date_created': 'dateCreated',
        'user_created': 'userCreated',
        'date_modified': 'dateModified',
        'user_modified': 'userModified'
    }

    def __init__(self, height=None, width=None, x_offset=None, y_offset=None, text_container=None, group_elements_id=None, group_element=None, ooxml_id=None, svg_blob_url=None, preset_type_id=None, free_form_path_xml=None, fill_map=None, is_theme_fill=None, effect=None, is_theme_effect=None, line=None, is_theme_line=None, flip_horizontal=None, flip_vertical=None, rotation=None, hidden=None, base_element_blob_url=None, changed_base_element_blob_url=None, package_uri=None, name=None, id=None, date_created=None, user_created=None, date_modified=None, user_modified=None):  # noqa: E501
        """SlideShapesDetails - a model defined in OpenAPI"""  # noqa: E501

        self._height = None
        self._width = None
        self._x_offset = None
        self._y_offset = None
        self._text_container = None
        self._group_elements_id = None
        self._group_element = None
        self._ooxml_id = None
        self._svg_blob_url = None
        self._preset_type_id = None
        self._free_form_path_xml = None
        self._fill_map = None
        self._is_theme_fill = None
        self._effect = None
        self._is_theme_effect = None
        self._line = None
        self._is_theme_line = None
        self._flip_horizontal = None
        self._flip_vertical = None
        self._rotation = None
        self._hidden = None
        self._base_element_blob_url = None
        self._changed_base_element_blob_url = None
        self._package_uri = None
        self._name = None
        self._id = None
        self._date_created = None
        self._user_created = None
        self._date_modified = None
        self._user_modified = None
        self.discriminator = None

        if height is not None:
            self.height = height
        if width is not None:
            self.width = width
        if x_offset is not None:
            self.x_offset = x_offset
        if y_offset is not None:
            self.y_offset = y_offset
        if text_container is not None:
            self.text_container = text_container
        self.group_elements_id = group_elements_id
        if group_element is not None:
            self.group_element = group_element
        if ooxml_id is not None:
            self.ooxml_id = ooxml_id
        self.svg_blob_url = svg_blob_url
        self.preset_type_id = preset_type_id
        self.free_form_path_xml = free_form_path_xml
        if fill_map is not None:
            self.fill_map = fill_map
        if is_theme_fill is not None:
            self.is_theme_fill = is_theme_fill
        if effect is not None:
            self.effect = effect
        if is_theme_effect is not None:
            self.is_theme_effect = is_theme_effect
        if line is not None:
            self.line = line
        if is_theme_line is not None:
            self.is_theme_line = is_theme_line
        if flip_horizontal is not None:
            self.flip_horizontal = flip_horizontal
        if flip_vertical is not None:
            self.flip_vertical = flip_vertical
        if rotation is not None:
            self.rotation = rotation
        if hidden is not None:
            self.hidden = hidden
        self.base_element_blob_url = base_element_blob_url
        self.changed_base_element_blob_url = changed_base_element_blob_url
        self.package_uri = package_uri
        self.name = name
        if id is not None:
            self.id = id
        if date_created is not None:
            self.date_created = date_created
        if user_created is not None:
            self.user_created = user_created
        if date_modified is not None:
            self.date_modified = date_modified
        if user_modified is not None:
            self.user_modified = user_modified

    @property
    def height(self):
        """Gets the height of this SlideShapesDetails.  # noqa: E501


        :return: The height of this SlideShapesDetails.  # noqa: E501
        :rtype: int
        """
        return self._height

    @height.setter
    def height(self, height):
        """Sets the height of this SlideShapesDetails.


        :param height: The height of this SlideShapesDetails.  # noqa: E501
        :type: int
        """

        self._height = height

    @property
    def width(self):
        """Gets the width of this SlideShapesDetails.  # noqa: E501


        :return: The width of this SlideShapesDetails.  # noqa: E501
        :rtype: int
        """
        return self._width

    @width.setter
    def width(self, width):
        """Sets the width of this SlideShapesDetails.


        :param width: The width of this SlideShapesDetails.  # noqa: E501
        :type: int
        """

        self._width = width

    @property
    def x_offset(self):
        """Gets the x_offset of this SlideShapesDetails.  # noqa: E501


        :return: The x_offset of this SlideShapesDetails.  # noqa: E501
        :rtype: int
        """
        return self._x_offset

    @x_offset.setter
    def x_offset(self, x_offset):
        """Sets the x_offset of this SlideShapesDetails.


        :param x_offset: The x_offset of this SlideShapesDetails.  # noqa: E501
        :type: int
        """

        self._x_offset = x_offset

    @property
    def y_offset(self):
        """Gets the y_offset of this SlideShapesDetails.  # noqa: E501


        :return: The y_offset of this SlideShapesDetails.  # noqa: E501
        :rtype: int
        """
        return self._y_offset

    @y_offset.setter
    def y_offset(self, y_offset):
        """Sets the y_offset of this SlideShapesDetails.


        :param y_offset: The y_offset of this SlideShapesDetails.  # noqa: E501
        :type: int
        """

        self._y_offset = y_offset

    @property
    def text_container(self):
        """Gets the text_container of this SlideShapesDetails.  # noqa: E501


        :return: The text_container of this SlideShapesDetails.  # noqa: E501
        :rtype: SharedTextContainerDetails
        """
        return self._text_container

    @text_container.setter
    def text_container(self, text_container):
        """Sets the text_container of this SlideShapesDetails.


        :param text_container: The text_container of this SlideShapesDetails.  # noqa: E501
        :type: SharedTextContainerDetails
        """

        self._text_container = text_container

    @property
    def group_elements_id(self):
        """Gets the group_elements_id of this SlideShapesDetails.  # noqa: E501


        :return: The group_elements_id of this SlideShapesDetails.  # noqa: E501
        :rtype: str
        """
        return self._group_elements_id

    @group_elements_id.setter
    def group_elements_id(self, group_elements_id):
        """Sets the group_elements_id of this SlideShapesDetails.


        :param group_elements_id: The group_elements_id of this SlideShapesDetails.  # noqa: E501
        :type: str
        """

        self._group_elements_id = group_elements_id

    @property
    def group_element(self):
        """Gets the group_element of this SlideShapesDetails.  # noqa: E501


        :return: The group_element of this SlideShapesDetails.  # noqa: E501
        :rtype: SlideGroupElementsDetails
        """
        return self._group_element

    @group_element.setter
    def group_element(self, group_element):
        """Sets the group_element of this SlideShapesDetails.


        :param group_element: The group_element of this SlideShapesDetails.  # noqa: E501
        :type: SlideGroupElementsDetails
        """

        self._group_element = group_element

    @property
    def ooxml_id(self):
        """Gets the ooxml_id of this SlideShapesDetails.  # noqa: E501


        :return: The ooxml_id of this SlideShapesDetails.  # noqa: E501
        :rtype: int
        """
        return self._ooxml_id

    @ooxml_id.setter
    def ooxml_id(self, ooxml_id):
        """Sets the ooxml_id of this SlideShapesDetails.


        :param ooxml_id: The ooxml_id of this SlideShapesDetails.  # noqa: E501
        :type: int
        """

        self._ooxml_id = ooxml_id

    @property
    def svg_blob_url(self):
        """Gets the svg_blob_url of this SlideShapesDetails.  # noqa: E501


        :return: The svg_blob_url of this SlideShapesDetails.  # noqa: E501
        :rtype: str
        """
        return self._svg_blob_url

    @svg_blob_url.setter
    def svg_blob_url(self, svg_blob_url):
        """Sets the svg_blob_url of this SlideShapesDetails.


        :param svg_blob_url: The svg_blob_url of this SlideShapesDetails.  # noqa: E501
        :type: str
        """

        self._svg_blob_url = svg_blob_url

    @property
    def preset_type_id(self):
        """Gets the preset_type_id of this SlideShapesDetails.  # noqa: E501


        :return: The preset_type_id of this SlideShapesDetails.  # noqa: E501
        :rtype: str
        """
        return self._preset_type_id

    @preset_type_id.setter
    def preset_type_id(self, preset_type_id):
        """Sets the preset_type_id of this SlideShapesDetails.


        :param preset_type_id: The preset_type_id of this SlideShapesDetails.  # noqa: E501
        :type: str
        """

        self._preset_type_id = preset_type_id

    @property
    def free_form_path_xml(self):
        """Gets the free_form_path_xml of this SlideShapesDetails.  # noqa: E501


        :return: The free_form_path_xml of this SlideShapesDetails.  # noqa: E501
        :rtype: str
        """
        return self._free_form_path_xml

    @free_form_path_xml.setter
    def free_form_path_xml(self, free_form_path_xml):
        """Sets the free_form_path_xml of this SlideShapesDetails.


        :param free_form_path_xml: The free_form_path_xml of this SlideShapesDetails.  # noqa: E501
        :type: str
        """

        self._free_form_path_xml = free_form_path_xml

    @property
    def fill_map(self):
        """Gets the fill_map of this SlideShapesDetails.  # noqa: E501


        :return: The fill_map of this SlideShapesDetails.  # noqa: E501
        :rtype: SharedFillMapDetails
        """
        return self._fill_map

    @fill_map.setter
    def fill_map(self, fill_map):
        """Sets the fill_map of this SlideShapesDetails.


        :param fill_map: The fill_map of this SlideShapesDetails.  # noqa: E501
        :type: SharedFillMapDetails
        """

        self._fill_map = fill_map

    @property
    def is_theme_fill(self):
        """Gets the is_theme_fill of this SlideShapesDetails.  # noqa: E501


        :return: The is_theme_fill of this SlideShapesDetails.  # noqa: E501
        :rtype: bool
        """
        return self._is_theme_fill

    @is_theme_fill.setter
    def is_theme_fill(self, is_theme_fill):
        """Sets the is_theme_fill of this SlideShapesDetails.


        :param is_theme_fill: The is_theme_fill of this SlideShapesDetails.  # noqa: E501
        :type: bool
        """

        self._is_theme_fill = is_theme_fill

    @property
    def effect(self):
        """Gets the effect of this SlideShapesDetails.  # noqa: E501


        :return: The effect of this SlideShapesDetails.  # noqa: E501
        :rtype: SharedEffectsDetails
        """
        return self._effect

    @effect.setter
    def effect(self, effect):
        """Sets the effect of this SlideShapesDetails.


        :param effect: The effect of this SlideShapesDetails.  # noqa: E501
        :type: SharedEffectsDetails
        """

        self._effect = effect

    @property
    def is_theme_effect(self):
        """Gets the is_theme_effect of this SlideShapesDetails.  # noqa: E501


        :return: The is_theme_effect of this SlideShapesDetails.  # noqa: E501
        :rtype: bool
        """
        return self._is_theme_effect

    @is_theme_effect.setter
    def is_theme_effect(self, is_theme_effect):
        """Sets the is_theme_effect of this SlideShapesDetails.


        :param is_theme_effect: The is_theme_effect of this SlideShapesDetails.  # noqa: E501
        :type: bool
        """

        self._is_theme_effect = is_theme_effect

    @property
    def line(self):
        """Gets the line of this SlideShapesDetails.  # noqa: E501


        :return: The line of this SlideShapesDetails.  # noqa: E501
        :rtype: SharedLinesDetails
        """
        return self._line

    @line.setter
    def line(self, line):
        """Sets the line of this SlideShapesDetails.


        :param line: The line of this SlideShapesDetails.  # noqa: E501
        :type: SharedLinesDetails
        """

        self._line = line

    @property
    def is_theme_line(self):
        """Gets the is_theme_line of this SlideShapesDetails.  # noqa: E501


        :return: The is_theme_line of this SlideShapesDetails.  # noqa: E501
        :rtype: bool
        """
        return self._is_theme_line

    @is_theme_line.setter
    def is_theme_line(self, is_theme_line):
        """Sets the is_theme_line of this SlideShapesDetails.


        :param is_theme_line: The is_theme_line of this SlideShapesDetails.  # noqa: E501
        :type: bool
        """

        self._is_theme_line = is_theme_line

    @property
    def flip_horizontal(self):
        """Gets the flip_horizontal of this SlideShapesDetails.  # noqa: E501


        :return: The flip_horizontal of this SlideShapesDetails.  # noqa: E501
        :rtype: bool
        """
        return self._flip_horizontal

    @flip_horizontal.setter
    def flip_horizontal(self, flip_horizontal):
        """Sets the flip_horizontal of this SlideShapesDetails.


        :param flip_horizontal: The flip_horizontal of this SlideShapesDetails.  # noqa: E501
        :type: bool
        """

        self._flip_horizontal = flip_horizontal

    @property
    def flip_vertical(self):
        """Gets the flip_vertical of this SlideShapesDetails.  # noqa: E501


        :return: The flip_vertical of this SlideShapesDetails.  # noqa: E501
        :rtype: bool
        """
        return self._flip_vertical

    @flip_vertical.setter
    def flip_vertical(self, flip_vertical):
        """Sets the flip_vertical of this SlideShapesDetails.


        :param flip_vertical: The flip_vertical of this SlideShapesDetails.  # noqa: E501
        :type: bool
        """

        self._flip_vertical = flip_vertical

    @property
    def rotation(self):
        """Gets the rotation of this SlideShapesDetails.  # noqa: E501


        :return: The rotation of this SlideShapesDetails.  # noqa: E501
        :rtype: int
        """
        return self._rotation

    @rotation.setter
    def rotation(self, rotation):
        """Sets the rotation of this SlideShapesDetails.


        :param rotation: The rotation of this SlideShapesDetails.  # noqa: E501
        :type: int
        """

        self._rotation = rotation

    @property
    def hidden(self):
        """Gets the hidden of this SlideShapesDetails.  # noqa: E501


        :return: The hidden of this SlideShapesDetails.  # noqa: E501
        :rtype: bool
        """
        return self._hidden

    @hidden.setter
    def hidden(self, hidden):
        """Sets the hidden of this SlideShapesDetails.


        :param hidden: The hidden of this SlideShapesDetails.  # noqa: E501
        :type: bool
        """

        self._hidden = hidden

    @property
    def base_element_blob_url(self):
        """Gets the base_element_blob_url of this SlideShapesDetails.  # noqa: E501


        :return: The base_element_blob_url of this SlideShapesDetails.  # noqa: E501
        :rtype: str
        """
        return self._base_element_blob_url

    @base_element_blob_url.setter
    def base_element_blob_url(self, base_element_blob_url):
        """Sets the base_element_blob_url of this SlideShapesDetails.


        :param base_element_blob_url: The base_element_blob_url of this SlideShapesDetails.  # noqa: E501
        :type: str
        """

        self._base_element_blob_url = base_element_blob_url

    @property
    def changed_base_element_blob_url(self):
        """Gets the changed_base_element_blob_url of this SlideShapesDetails.  # noqa: E501


        :return: The changed_base_element_blob_url of this SlideShapesDetails.  # noqa: E501
        :rtype: str
        """
        return self._changed_base_element_blob_url

    @changed_base_element_blob_url.setter
    def changed_base_element_blob_url(self, changed_base_element_blob_url):
        """Sets the changed_base_element_blob_url of this SlideShapesDetails.


        :param changed_base_element_blob_url: The changed_base_element_blob_url of this SlideShapesDetails.  # noqa: E501
        :type: str
        """

        self._changed_base_element_blob_url = changed_base_element_blob_url

    @property
    def package_uri(self):
        """Gets the package_uri of this SlideShapesDetails.  # noqa: E501


        :return: The package_uri of this SlideShapesDetails.  # noqa: E501
        :rtype: str
        """
        return self._package_uri

    @package_uri.setter
    def package_uri(self, package_uri):
        """Sets the package_uri of this SlideShapesDetails.


        :param package_uri: The package_uri of this SlideShapesDetails.  # noqa: E501
        :type: str
        """

        self._package_uri = package_uri

    @property
    def name(self):
        """Gets the name of this SlideShapesDetails.  # noqa: E501


        :return: The name of this SlideShapesDetails.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this SlideShapesDetails.


        :param name: The name of this SlideShapesDetails.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def id(self):
        """Gets the id of this SlideShapesDetails.  # noqa: E501


        :return: The id of this SlideShapesDetails.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SlideShapesDetails.


        :param id: The id of this SlideShapesDetails.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def date_created(self):
        """Gets the date_created of this SlideShapesDetails.  # noqa: E501


        :return: The date_created of this SlideShapesDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_created

    @date_created.setter
    def date_created(self, date_created):
        """Sets the date_created of this SlideShapesDetails.


        :param date_created: The date_created of this SlideShapesDetails.  # noqa: E501
        :type: datetime
        """

        self._date_created = date_created

    @property
    def user_created(self):
        """Gets the user_created of this SlideShapesDetails.  # noqa: E501


        :return: The user_created of this SlideShapesDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_created

    @user_created.setter
    def user_created(self, user_created):
        """Sets the user_created of this SlideShapesDetails.


        :param user_created: The user_created of this SlideShapesDetails.  # noqa: E501
        :type: str
        """

        self._user_created = user_created

    @property
    def date_modified(self):
        """Gets the date_modified of this SlideShapesDetails.  # noqa: E501


        :return: The date_modified of this SlideShapesDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_modified

    @date_modified.setter
    def date_modified(self, date_modified):
        """Sets the date_modified of this SlideShapesDetails.


        :param date_modified: The date_modified of this SlideShapesDetails.  # noqa: E501
        :type: datetime
        """

        self._date_modified = date_modified

    @property
    def user_modified(self):
        """Gets the user_modified of this SlideShapesDetails.  # noqa: E501


        :return: The user_modified of this SlideShapesDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_modified

    @user_modified.setter
    def user_modified(self, user_modified):
        """Sets the user_modified of this SlideShapesDetails.


        :param user_modified: The user_modified of this SlideShapesDetails.  # noqa: E501
        :type: str
        """

        self._user_modified = user_modified

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SlideShapesDetails):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

# flake8: noqa

"""
    Story

    This API is the main entry point for creating, editing and publishing analytics throught the Presalytics API  # noqa: E501

    The version of the OpenAPI document: 0.3.1
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from presalytics.client.presalytics_story.api.default_api import DefaultApi

# import ApiClient
from presalytics.client.presalytics_story.api_client import ApiClient
from presalytics.client.presalytics_story.configuration import Configuration
from presalytics.client.presalytics_story.exceptions import OpenApiException
from presalytics.client.presalytics_story.exceptions import ApiTypeError
from presalytics.client.presalytics_story.exceptions import ApiValueError
from presalytics.client.presalytics_story.exceptions import ApiKeyError
from presalytics.client.presalytics_story.exceptions import ApiException
# import models into sdk package
from presalytics.client.presalytics_story.models.add_new_collaborator_request import AddNewCollaboratorRequest
from presalytics.client.presalytics_story.models.base_model import BaseModel
from presalytics.client.presalytics_story.models.cache_post_request import CachePostRequest
from presalytics.client.presalytics_story.models.collaborator_bulk_update_request import CollaboratorBulkUpdateRequest
from presalytics.client.presalytics_story.models.create_session_request import CreateSessionRequest
from presalytics.client.presalytics_story.models.event import Event
from presalytics.client.presalytics_story.models.event_all_of import EventAllOf
from presalytics.client.presalytics_story.models.file_upload import FileUpload
from presalytics.client.presalytics_story.models.manage_event import ManageEvent
from presalytics.client.presalytics_story.models.message import Message
from presalytics.client.presalytics_story.models.modify_inactive_collaborator import ModifyInactiveCollaborator
from presalytics.client.presalytics_story.models.ooxml_document import OoxmlDocument
from presalytics.client.presalytics_story.models.ooxml_document_all_of import OoxmlDocumentAllOf
from presalytics.client.presalytics_story.models.outline import Outline
from presalytics.client.presalytics_story.models.permission_type import PermissionType
from presalytics.client.presalytics_story.models.permission_type_all_of import PermissionTypeAllOf
from presalytics.client.presalytics_story.models.problem_detail import ProblemDetail
from presalytics.client.presalytics_story.models.required_parameters_to_create_a_view import RequiredParametersToCreateAView
from presalytics.client.presalytics_story.models.session import Session
from presalytics.client.presalytics_story.models.session_all_of import SessionAllOf
from presalytics.client.presalytics_story.models.status import Status
from presalytics.client.presalytics_story.models.story import Story
from presalytics.client.presalytics_story.models.story_all_of import StoryAllOf
from presalytics.client.presalytics_story.models.story_collaborator import StoryCollaborator
from presalytics.client.presalytics_story.models.story_collaborator_all_of import StoryCollaboratorAllOf
from presalytics.client.presalytics_story.models.story_outline_history import StoryOutlineHistory
from presalytics.client.presalytics_story.models.story_outline_history_all_of import StoryOutlineHistoryAllOf
from presalytics.client.presalytics_story.models.view import View
from presalytics.client.presalytics_story.models.view_all_of import ViewAllOf





import typing
import base64
import uuid
import logging
import jinja2
import json
import os
import lxml
import lxml.html
import presalytics
import presalytics.lib.exceptions

if typing.TYPE_CHECKING:
    pass


logger = logging.getLogger(__name__)


class DataTableWidget(presalytics.story.components.WidgetBase):
    """
    A `Widget` for rendering data tables, implemented over the top of bootstrap-table.
    For security reasons, data tables are loaded via a sandboxed iframe that points
    to an endpoint in the Presalytics Story API.

    This class allows users to load [bootstrap-table](https://bootstrap-table.com) objects
    into widgets in order to create responsive data tables directly off a simple data object.

    Parameters
    ----------

    name : str
        the name of widget.  Must be unique within `presalytics.COMPONENTS`

    data: dict
        Data that will be loaded into the `c3.generate()` method.  Please go
        to [c3js.org](https://c3js.org/gettingstarted.html) for more information 
        on how to configure this object.

    css64 : str. optional
        A base64-encoded string of the css styles to apply to the d3 document.  Used for server-to-server
        transport over https.
    
    css_filename: str, optional
        A css file containing styles that will be applied to d3

        Note: Styles `html {width: 100%; height:100%;} body {width: 100%; height: 100%; margin: 0px;}`
        are applied by default if not css is provided

    """
    __component_kind__ = 'data-table'
 

    def __init__(self, 
                 name: str,
                 table_data: typing.Dict,
                 css64: str = None,
                 css_filename: str = None, 
                 *args,
                 **kwargs):
        self.table_data = table_data
        super(DataTableWidget, self).__init__(name, *args, **kwargs)
        self.css64 = self.read_file(css_filename)
        self.css64 = self.css64 if self.css64 else css64
        self.css_filename = css_filename

    def read_file(self, filename) -> typing.Optional[str]:
        """
        Find a file named `filename` and return its base64-ecoded content
        """
        data64 = None
        if filename:
            search_paths = list(set(presalytics.autodiscover_paths))
            if os.getcwd() not in search_paths:
                search_paths.append(os.getcwd())
            for path in search_paths:
                fpath = os.path.join(path, filename)
                if os.path.exists(fpath):
                    with open(fpath, 'rb') as f:
                        data = f.read()
                        data64 = base64.b64encode(data).decode('utf-8')  #type: ignore
                    break
            if not data64:
                logger.debug("File {0} could not be found".format(filename))
        return data64


    def to_html(self, data=None, **kwargs) -> str:
        """
        Renders the sandboxed iframe with will contain the d3 script widget
        """
        html = self.create_container()
        return html

    def create_container(self, **kwargs):
        """
        Wraps the D3 objects in an endpoint at the story API load via a sandboxed `<iframe>` that
        will be rendered
        """
        story_host = self.get_client(delegate_login=True).story.api_client.external_root_url
        source_url = "{0}/cache/{1}".format(story_host, self.nonce)
        empty_parent_div = lxml.html.Element("div", {
            'class': 'empty-parent bg-light',
            'style': 'height: 100%; width: 100%, display: block; text-align: left;'
        })
        frame = lxml.html.Element("iframe", {
            'src': source_url,
            'frameborder': "0",
            'scrolling': "auto",
            'class': 'd3-responsive-frame',
            'style': 'max-height: none; max-width: none; height:100%; width: 100%;',
            'sandbox': 'allow-scripts'
        })
        empty_parent_div.append(frame)
        return lxml.html.tostring(empty_parent_div).decode('utf-8')

    @classmethod
    def deserialize(cls, outline, **kwargs):
        table_data = outline.data.get("table_data")
        return cls(outline.name,
                   table_data=table_data,
                    **kwargs)

    def serialize(self, **kwargs):
        data = {
            'table_data': self.table_data,
        }
        return presalytics.story.outline.Widget(
            name=self.name,
            kind=self.__component_kind__,
            data=data,
        )

    DEFAULT_CSS = """
    html {
        height: 100%;
        width: 100%;
    }
    body {
        height: 100%;
        width: 100%;
        margin: 0px;
        background-color: #f8f9fa !important;
    }

    .data-table-container {
        border: none !important;
    }

    .data-table-container tbody {
        background-color: #fff;
    }

    .data-table-container thead {
        background-color: #199ec7;
        color: white;
    }


    """

    def create_subdocument(self, **kwargs):
        return self.standalone_html()

    def standalone_html(self) -> str:
        """
        Returns string with an html document containing that d3 widget

        Loaded via the Story API d3 endpoint
        """
        SIMPLE_HTML = jinja2.Template("""<!DOCTYPE html>
        <html>
            <head>
                <meta charset="UTF-8">
                <style>
                {{ css }}
                </style>
            </head>
            <body>
                <link href="{{bootstrap4_css_url}}" rel="stylesheet"/>
                <link href="{{font_awesome_url}}" rel="stylesheet"/>
                <link href="{{bootstrap_table_css_url}}" rel="stylesheet"/>
                <div id="table" class="data-table-container"></div>
                <script type="text/javascript" src="{{ jquery_url }}"></script>
                <script type="text/javascript" src="{{ popper_url }}"></script>  
                <script type="text/javascript" src="{{ bootstrap4_js_url }}"></script>  
                <script type="text/javascript" src="{{ boostrap_table_js_url }}"></script>  
                <script type="text/javascript">
                    $(document).ready(function() {
                        var data = JSON.parse('{{data|safe}}');
                        $("#table").bootstrapTable(data);
                    });
                </script>
            </body>
        </html>""")
        data = json.dumps(self.table_data)  # dont use hyphens in data keys
        extra_css = base64.b64decode(self.css64).decode('utf-8') if self.css64 else DataTableWidget.DEFAULT_CSS  #type: ignore  
        context = {
            "bootstrap4_css_url": presalytics.lib.plugins.external.ApprovedExternalLinks().attr_dict.flatten().get('bootstrap4'),
            "font_awesome_url": presalytics.lib.plugins.external.ApprovedExternalLinks().attr_dict.flatten().get('font-awesome'),
            "bootstrap_table_css_url": presalytics.lib.plugins.external.ApprovedExternalLinks().attr_dict.flatten().get('bootstrap-table'),
            "jquery_url": presalytics.lib.plugins.external.ApprovedExternalScripts().attr_dict.flatten().get('jquery'),
            "popper_url": presalytics.lib.plugins.external.ApprovedExternalScripts().attr_dict.flatten().get('popper'),
            "boostrap_table_js_url": presalytics.lib.plugins.external.ApprovedExternalScripts().attr_dict.flatten().get('bootstrap-table'),
            "d3_url": presalytics.lib.plugins.external.ApprovedExternalScripts().attr_dict.flatten().get('d3'),
            "data": data,
            "css": extra_css
        }
        return SIMPLE_HTML.render(**context)



# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class SharedGradientStops(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'gradient_fills_id': 'str',
        'position': 'int',
        'id': 'str'
    }

    attribute_map = {
        'gradient_fills_id': 'gradientFillsId',
        'position': 'position',
        'id': 'id'
    }

    def __init__(self, gradient_fills_id=None, position=None, id=None):  # noqa: E501
        """SharedGradientStops - a model defined in OpenAPI"""  # noqa: E501

        self._gradient_fills_id = None
        self._position = None
        self._id = None
        self.discriminator = None

        self.gradient_fills_id = gradient_fills_id
        if position is not None:
            self.position = position
        if id is not None:
            self.id = id

    @property
    def gradient_fills_id(self):
        """Gets the gradient_fills_id of this SharedGradientStops.  # noqa: E501


        :return: The gradient_fills_id of this SharedGradientStops.  # noqa: E501
        :rtype: str
        """
        return self._gradient_fills_id

    @gradient_fills_id.setter
    def gradient_fills_id(self, gradient_fills_id):
        """Sets the gradient_fills_id of this SharedGradientStops.


        :param gradient_fills_id: The gradient_fills_id of this SharedGradientStops.  # noqa: E501
        :type: str
        """

        self._gradient_fills_id = gradient_fills_id

    @property
    def position(self):
        """Gets the position of this SharedGradientStops.  # noqa: E501


        :return: The position of this SharedGradientStops.  # noqa: E501
        :rtype: int
        """
        return self._position

    @position.setter
    def position(self, position):
        """Sets the position of this SharedGradientStops.


        :param position: The position of this SharedGradientStops.  # noqa: E501
        :type: int
        """

        self._position = position

    @property
    def id(self):
        """Gets the id of this SharedGradientStops.  # noqa: E501


        :return: The id of this SharedGradientStops.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SharedGradientStops.


        :param id: The id of this SharedGradientStops.  # noqa: E501
        :type: str
        """

        self._id = id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SharedGradientStops):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other




import urllib.parse
import requests
import posixpath
import logging
import typing
import webbrowser
import json
import time
import jose
import os
import jose.jwt
import cachetools
import presalytics.lib
import presalytics.lib.exceptions
import presalytics.lib.constants as cnst


logger = logging.getLogger(__name__)


@cachetools.cached(cache=cachetools.TTLCache(maxsize=4096, ttl=300))  # Cache result for 5 minutes
def get_jwks():
    auth_host = os.environ.get("OIDC_AUTH_HOST", cnst.OIDC_AUTH_HOST)
    jwks_path = os.environ.get("jwks_path", ".well-known/jwks.json")
    jwks_endpoint = posixpath.join(auth_host, jwks_path)
    r = requests.get(jwks_endpoint)
    if r.status_code == 200:
        jwks = r.json()
        logger.debug('Updated Json Web Key Set from {}'.format(jwks_endpoint))
        return jwks
    else:
        raise presalytics.lib.exceptions.ApiError(message="Could not get jwks from Uri", status_code=r.status_code)



class OidcClient(object):
    """
    A helper class for negotiating tokens from an oidc provider, defalting to https://login.presalytics.io

    Parameters
    ----------

    client_id: str, optional
        The client id for the application requesting a token. Defaults to python-client

    client_secret: str, optional
        The client secret for the application requesting a token.  Defaults to None (python-client
        is a public facing client with lower priviledge on the Presalytics API). Contact presalytics
        if need a higher-privleged client to access the Presalytics API

    audience: str, optional
        The default audience for client request.  Defaults to https://api.presalytics.io/

    validate_tokens: bool
        Whether to validate tokens when recieved from the token endpoint.  Defaults to True.


    """
    def __init__(self, client_id=None, client_secret=None, validate_tokens=True, *args, **kwargs):
        self.auth_host = kwargs.get("auth_host", cnst.OIDC_AUTH_HOST)
        self.well_known_endpoint = posixpath.join(self.auth_host, kwargs.get("well_known_path", ".well-known/openid-configuration"))
        self.token_endpoint = posixpath.join(self.auth_host, kwargs.get("token_path", "oauth/token"))
        self.authorization_endpoint = posixpath.join(self.auth_host, kwargs.get("authorization_path", "authorization"))
        self.device_endpoint = posixpath.join(self.auth_host, kwargs.get("device_path", "oauth/device/code"))
        self.jwks_endpoint = posixpath.join(self.auth_host, kwargs.get("jwks_path", ".well-known/jwks.json"))
        self.userinfo_endpoint = posixpath.join(self.auth_host, kwargs.get("userinfo_path", "userinfo"))
        self.client_id = client_id if client_id else cnst.DEFAULT_CLIENT_ID
        self.audience = kwargs.get("audience", cnst.DEFAULT_AUDIENCE)
        self.client_secret = client_secret
        self.default_scopes = "openid email profile offline_access"
        self.validate_tokens = validate_tokens
        self.repoll_errors = [
            "authorization_pending",
            "slow_down"
        ]

    def token(self, username, password=None, audience=None, scope=None, **kwargs) -> typing.Dict:
        """
        Get an access token
        """
        if not scope:
            scope = self.default_scopes
        if not audience:
            audience = self.audience
        if password and self.client_secret:
            #use password grant if present (not recommended)
            data = {
                "grant_type": "password",
                "username": username,
                "password": password,
                "audience": audience,
                "client_id": self.client_id,
                "client_secret": self.client_secret,
                "scope": scope
            }
            token_data = self._post(self.token_endpoint, data)
            
        else:
            # Use device grant as default
            device_data = {
                'client_id': self.client_id,
                'audience': self.audience,
                'scope': scope
            }

            device_code_response = self._post(self.device_endpoint, device_data)
            user_code_message = "This device's user code is: {}.  Please verify this code when logging in.".format(device_code_response["user_code"])
            print(user_code_message)
            cli_message = "Please open a webrowser to {0} and login.".format(device_code_response["verification_uri_complete"])
            print(cli_message)
            try:
                webbrowser.open_new_tab(device_code_response["verification_uri_complete"])
            except:
                pass
            sleep_interval = device_code_response["interval"]
            auth_data = {
                "grant_type": "urn:ietf:params:oauth:grant-type:device_code",
                "device_code": device_code_response["device_code"],
                "client_id": self.client_id
            }
            headers = {
                'content-type': 'application/x-www-form-urlencoded'
            }
            repoll = True
            while repoll:
                token_response = requests.post(self.token_endpoint, auth_data, headers=headers)
                if token_response.status_code != 200:
                    err_resp = token_response.json()
                    err_msg = err_resp["error"]
                    if err_msg in self.repoll_errors:
                        time.sleep(sleep_interval)
                        if err_msg == "slow_down":
                            time.sleep(sleep_interval)
                        logger.debug("User has not yet logged in.  Repolling...")                     
                    else:
                        message = "Error: {0} -- {1}".format(err_msg, err_resp["error_description"])
                        raise presalytics.lib.exceptions.ApiError(message=message, status_code=token_response.status_code)
                else:
                    repoll = False
            token_data = token_response.json()
            if token_data.get('access_token', None):
                print("Login Success! Please continue with your work.")
                logger.debug("User logged in successfully.")
            else:
                message = "Error: {0} -- {1}".format(err_msg, err_resp["error_description"])
                raise presalytics.lib.exceptions.ApiError(message=message, status_code=token_response.status_code)
        if self.validate_tokens:
            self.validate_token(token_data["access_token"])
        return token_data

    def validate_token(self, token):
        """
        Validate a token
        """
        unverified_header = jose.jwt.get_unverified_header(token)
        rsa_key = {}
        jwks = get_jwks()
        for key in jwks["keys"]:
            if key["kid"] == unverified_header["kid"]:
                rsa_key = {
                    "kty": key["kty"],
                    "kid": key["kid"],
                    "use": key["use"],
                    "n": key["n"],
                    "e": key["e"]
                }
        if rsa_key:
            try:
                payload = jose.jwt.decode(
                    token,
                    rsa_key,
                    algorithms=["RS256"],
                    audience=self.audience,
                    issuer=self.auth_host
                )
            except jose.jwt.ExpiredSignatureError:
                raise presalytics.lib.exceptions.ApiError(message="token_expired", status_code=401)
            except jose.jwt.JWTClaimsError:
                raise presalytics.lib.exceptions.ApiError(message="invalid_claims: check audience and issuer", status_code=401)
            except Exception:
                raise presalytics.lib.exceptions.ApiError(message="invalid token (likely malformed)", status_code=401)
            logger.debug("Access token validated.")
            return payload
        
        raise presalytics.lib.exceptions.ApiError(message="invalid_header: could not find key in jwks",status_code=401)
        


    def refresh_token(self, refresh_token, scope=None):
        """
        Exchange a refresh token for an access token
        """
        if not scope:
            scope = self.default_scopes
        if not self.client_secret:
            raise presalytics.lib.ApiError(message="Cannot refresh token without client secret", status_code=401)
        data = {
            "grant_type": "refresh_token",
            "client_id": self.client_id,
            "refresh_token": refresh_token,
            "scope": scope,
            "audience": self.audience,
            "client_secret": self.client_secret
        }

        token_data = self._post(self.token_endpoint, data)
        
        if self.validate_tokens:
            self.validate_token(token_data["access_token"])
        return token_data

    def _post(self, endpoint, data, headers={}):
        try:
            if 'content-type' not in [x.lower() for x in headers.keys()]:
                headers.update({'content-type': 'application/x-www-form-urlencoded'})
            response = requests.post(endpoint, data, headers=headers)
        except Exception as ex:
            logger.exception(ex)
            raise ex

        return self._handle_response(response)


    def _handle_response(self, response):
        if response.status_code == 401:
            raise presalytics.lib.exceptions.ApiError(message="Unauthorized", status_code=401)
        elif response.status_code == 403:
            raise presalytics.lib.exceptions.ApiError(message="Forbidden", status_code=403)
        elif response.status_code == 409:
            logger.error("Value already exists")
            data = None
        elif response.status_code > 299:
            try:
                message = response.json()['message']
            except (KeyError, ValueError):
                message = response.content
            raise presalytics.lib.exceptions.ApiError(message=message, status_code=response.status_code)
        elif response.status_code == 204:
            data = None
        elif response.status_code == 200:
            try:
                data = response.json()
            except ValueError:
                try:
                    data = response.content.decode('utf-8')
                except Exception:
                    data = response.content
                try:
                    data = json.loads(data)
                except Exception:
                    pass
        return data
    
    def client_credentials_token(self, audience=None, scope=None):
        if not self.client_secret:
            raise presalytics.lib.exceptions.ApiError(message="Must have client secret for client credentials grant", status_code=400)
        if not audience:
            audience = self.audience
        if not scope:
            scope = self.default_scopes
        post_data = {
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "grant_type": "client_credentials",
            "audience": audience
        }
        return self._post(self.token_endpoint, post_data)

    def get_user_id(self, token) -> str:
        payload = presalytics.client.oidc.OidcClient().validate_token(token)
        return payload.get('https://api.presalytics.io/api_user_id', None)

    
        


        



        


import os
import sass
import typing
import presalytics.lib.exceptions
import presalytics.lib.plugins.base

class ScssPlugin(presalytics.lib.plugins.base.StylePlugin):
    """
    Compiles scss from files and a `dict` of variables and loads
    them into a script tag

    *Note: Any `<script>` tags nested into the resultant style tag will
    be removed at render-time by a `presalytics.story.components.Renderer`.
    These script tags will not make it to the browser.

    Configuration Values
    ----------
    filenames : list of str
        A list of scss filenames that will get compiled to css if
        all files are present

    rendered_css : str, optional
        If the files are not present, this string will be placed in the
        resultant style tag

    variables : dict, optional
        Scss variables to include. Dictionary keys should be prefixed with a
        `$` to indicate that they are scss variable names.  
    """
    __plugin_name__ = "scss_files"

    def __init__(self, **kwargs):
        super(ScssPlugin, self).__init__(**kwargs)

    def _files_are_local(self, filenames):
        for fname in filenames:
            if not os.path.exists(fname):
                return False
        return True

    def _create_variables_string(self, variables):
        _vars = ""
        for key, val in variables.items():
            _vars = _vars + key + ": " + val + "; "
        return _vars

    def _load_files_to_string(self, filenames):
        scss_string = ""
        for fname in filenames:
            with open(fname, 'r') as f:
                addl_scss = f.read()
            scss_string = scss_string + addl_scss
        return scss_string

    def make_css(self, filenames: typing.List[str], variables: typing.Dict[str, str]):
        """
        Compiles the scss in filenames to an html fragment using variables 
        
        Returns
        ----------
        A `<style>` tag html fragment in a string
        """
        _scss = self._create_variables_string(variables) + self._load_files_to_string(filenames)
        return sass.compile(string=_scss)

    def to_style(self, config, **kwargs):
        """
        Renders a config containing `Configuration Values` to a style tag
        """
        filenames = config.get("filenames", None)
        if not filenames:
            raise presalytics.lib.exceptions.MissingConfigException("Filenames are required in the config.")
        variables = config.get("variables", {})
        if self._files_are_local(filenames):
            rendered_css = self.make_css(filenames, variables)
        elif config.get("rendered_css", None):
            rendered_css = config.get("rendered_css")
        else:
            raise presalytics.lib.exceptions.MissingConfigException("A file referenced by this plugin could not be found.")
        return "<style>{0}</style>".format(rendered_css)


    @classmethod
    def configure(cls, filenames: typing.List[str], variables: typing.Dict = {}, old_css: str = None) -> typing.Dict:
        """
        Call this method when serializing a subclass of `presalytics.story.components.ComponentBase` 
        to generate a configuration for the plugin with a valid `rendered_css` entry
        """
        inst = cls()
        if not filenames:
            raise presalytics.lib.exceptions.MissingConfigException("Filenames are required in the config.")
        if inst._files_are_local(filenames):
            rendered_css = inst.make_css(filenames, variables)
        elif old_css:
            rendered_css = old_css
        else:
            raise presalytics.lib.exceptions.MissingConfigException("A file referenced by this plugin could not be found.")
        ret = {
            "name": inst.__plugin_name__,
            "kind": inst.__plugin_kind__,
            "config": {
                "filenames": filenames,
                "variables": variables,
                "rendered_css": rendered_css
            }
        }
        return ret




# coding: utf-8

"""
    Story

    This API is the main entry point for creating, editing and publishing analytics throught the Presalytics API  # noqa: E501

    The version of the OpenAPI document: 0.3.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class Message(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'body': 'str',
        'id': 'str',
        'remote': 'bool',
        'timestamp': 'datetime',
        'user_id': 'str',
        'user_name': 'str'
    }

    attribute_map = {
        'body': 'body',
        'id': 'id',
        'remote': 'remote',
        'timestamp': 'timestamp',
        'user_id': 'userId',
        'user_name': 'userName'
    }

    def __init__(self, body=None, id=None, remote=None, timestamp=None, user_id=None, user_name=None):  # noqa: E501
        """Message - a model defined in OpenAPI"""  # noqa: E501

        self._body = None
        self._id = None
        self._remote = None
        self._timestamp = None
        self._user_id = None
        self._user_name = None
        self.discriminator = None

        if body is not None:
            self.body = body
        if id is not None:
            self.id = id
        if remote is not None:
            self.remote = remote
        if timestamp is not None:
            self.timestamp = timestamp
        if user_id is not None:
            self.user_id = user_id
        if user_name is not None:
            self.user_name = user_name

    @property
    def body(self):
        """Gets the body of this Message.  # noqa: E501


        :return: The body of this Message.  # noqa: E501
        :rtype: str
        """
        return self._body

    @body.setter
    def body(self, body):
        """Sets the body of this Message.


        :param body: The body of this Message.  # noqa: E501
        :type: str
        """

        self._body = body

    @property
    def id(self):
        """Gets the id of this Message.  # noqa: E501


        :return: The id of this Message.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Message.


        :param id: The id of this Message.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def remote(self):
        """Gets the remote of this Message.  # noqa: E501


        :return: The remote of this Message.  # noqa: E501
        :rtype: bool
        """
        return self._remote

    @remote.setter
    def remote(self, remote):
        """Sets the remote of this Message.


        :param remote: The remote of this Message.  # noqa: E501
        :type: bool
        """

        self._remote = remote

    @property
    def timestamp(self):
        """Gets the timestamp of this Message.  # noqa: E501


        :return: The timestamp of this Message.  # noqa: E501
        :rtype: datetime
        """
        return self._timestamp

    @timestamp.setter
    def timestamp(self, timestamp):
        """Sets the timestamp of this Message.


        :param timestamp: The timestamp of this Message.  # noqa: E501
        :type: datetime
        """

        self._timestamp = timestamp

    @property
    def user_id(self):
        """Gets the user_id of this Message.  # noqa: E501


        :return: The user_id of this Message.  # noqa: E501
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this Message.


        :param user_id: The user_id of this Message.  # noqa: E501
        :type: str
        """

        self._user_id = user_id

    @property
    def user_name(self):
        """Gets the user_name of this Message.  # noqa: E501


        :return: The user_name of this Message.  # noqa: E501
        :rtype: str
        """
        return self._user_name

    @user_name.setter
    def user_name(self, user_name):
        """Sets the user_name of this Message.


        :param user_name: The user_name of this Message.  # noqa: E501
        :type: str
        """

        self._user_name = user_name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Message):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import os
import logging
import time
import posixpath
import urllib.parse
import flask
import threading
import webbrowser
import requests
import presalytics.lib.util
import presalytics.lib.exceptions


logger = logging.getLogger(__name__)


app = flask.Flask(__name__)


@app.route('/story/<id>')
def story(id):
    template_name = str(id) + '.html'
    return flask.render_template(template_name)

@app.route('/img/<path:filename>/')
def static_img(filename=None):
    directory = os.path.join(app.static_folder, "img")
    return flask.send_from_directory(directory, filename)


@app.route('/js/<path:filename>/')
def static_js(filename=None):
    directory = os.path.join(app.static_folder, "js")
    return flask.send_from_directory(directory, filename)


@app.route('/css/<path:filename>/')
def static_css(filename=None):
    directory = os.path.join(app.static_folder, "css")
    return flask.send_from_directory(directory, filename)


@app.route('/favicon.ico')
def favicon():
    return flask.send_from_directory(app.static_folder, 'favicon.ico')


@app.route('/shutdown', methods=['POST'])
def shutdown():
    shutdown_func = flask.request.environ.get('werkzeug.server.shutdown')
    shutdown_func()
    return 'Server shutting down...'


class LocalServer(object):
    def __init__(self, 
                 host='127.0.0.1', 
                 debug=True, 
                 port=8082, 
                 root_path=None,
                 use_reloader=False,
                 **kwargs):
        self.host = host
        self.debug = debug
        self.port = port
        self.root_path = self.make_local_folders(files_path=root_path)
        self.static_dir = os.path.join(self.root_path, "static")
        self.use_reloader = use_reloader
        self.get_preloaders()

    def run(self):
        app.root_path = self.root_path
        app.static_folder = self.static_dir
        app.run(host=self.host, 
                debug=self.debug, 
                port=self.port, 
                use_reloader=self.use_reloader)
    
    def make_local_folders(self, files_path=None):
        if files_path is None:
            files_path = os.getcwd()
        if not os.path.exists(files_path):
            os.mkdir(files_path)
        root = os.path.join(files_path, "presalytics")
        if not os.path.exists(root):
            os.mkdir(root)
        templates_path = os.path.join(root, "templates")
        if not os.path.exists(templates_path):
            os.mkdir(templates_path)
        static_files_path = os.path.join(root, "static")
        if not os.path.exists(static_files_path):
            os.mkdir(static_files_path)
        css_path = os.path.join(static_files_path, "css")
        if not os.path.exists(css_path):
            os.mkdir(css_path)
        js_path = os.path.join(static_files_path, "js")
        if not os.path.exists(js_path):
            os.mkdir(js_path)
        theme_path = os.path.join(css_path, "theme")
        if not os.path.exists(theme_path):
            os.mkdir(theme_path)
        img_path = os.path.join(static_files_path, "img")
        if not os.path.exists(img_path):
            os.mkdir(img_path)
        local_preloader_path = os.path.join(static_files_path, "preloaders")
        if not os.path.exists(local_preloader_path):
            os.mkdir(local_preloader_path)
        return root

    def get_preloaders(self):
        preloaders = [
            'bars.svg'
        ]
        host = presalytics.lib.util.get_site_host()
        local_preloader_path = os.path.join(self.static_dir, "preloaders")
        for img in preloaders:
            try:
                preloader_url = urllib.parse.urljoin(host, "/static/preloaders/" + img)
                response = requests.get(preloader_url)
                if response.status_code == 200:
                    filename = os.path.join(local_preloader_path, img)
                    open(filename, 'wb').write(response.content)
                else:
                    message = "Error downloading preloader " + img
                    raise presalytics.lib.exceptions.ApiError(message=message, status_code=response.status_code)
            except Exception as ex:
                message = "Could not download preloader {0} from {1}".format(img, host)
                logger.error(message)
                # logger.exception(ex) # non-critical error



    def get_static_files_dict(self):
        static_files_dict = {}
        root = self.root_path
        static = os.path.join(root, "static")
        for item in os.listdir(static):
            if os.path.isdir(item):
                key = os.path.basename(item)
                value = os.path.abspath(item)
                static_files_dict[key] = value
        static_files_dict['favicon.ico'] = os.path.join(static, "favicon.ico")
        return static_files_dict
                


class Browser(threading.Thread):
    def __init__(self, address, delay_time=2, *args, **kwargs):
        super(Browser, self).__init__(*args, **kwargs)
        self.address = address
        self.delay_time = delay_time

    def run(self):
        time.sleep(self.delay_time)
        webbrowser.open_new_tab(self.address)


if __name__ == '__main__':
    app.run(debug=True)




import typing
import base64
import uuid
import logging
import jinja2
import json
import os
import lxml
import lxml.html
import markdown
import mdx_gfm
import presalytics
import presalytics.lib.exceptions


if typing.TYPE_CHECKING:
    pass


logger = logging.getLogger(__name__)


class MarkdownWidget(presalytics.story.components.WidgetBase):
    """
    A `Widget` for rendering markdown to html objects.

    This implementation uses [Github-flavored Markdown](https://github.github.com/gfm/).

    Parameters
    ----------

    name : str
        the name of widget.  Must be unique within `presalytics.COMPONENTS`

    markdown_text: str
        A string of markdown text to be converted to html via the `to_html` method

    css64 : str. optional
        A base64-encoded string of the css styles to apply to the d3 document.  Used for server-to-server
        transport over https.
    
    css_filename: str, optional
        A css file containing styles that will be applied to d3

        Note: Styles `html {width: 100%; height:100%;} body {width: 100%; height: 100%; margin: 0px;}`
        are applied by default if not css is provided


    Attributes
    -----------

    md: `markdown.Markdown`
        A markdown renderer. By default this rendered loads `py-gfm`'s
        `mdx_gfm.GithubFlavoredMarkdownExtension` to extend the renderer

    """
    __component_kind__ = 'markdown'
    __plugins__ = [
        {
            'name': 'external_links',
            'kind': 'style',
            'config': {
                'approved_styles_key': 'markdown-css'
            }
        }
    ]

    def __init__(self, 
                 name: str,
                 markdown_text: str,
                 css64: str = None,
                 css_filename: str = None, 
                 *args,
                 **kwargs):
        self.markdown_text = markdown_text
        self.md = markdown.Markdown(extensions=[mdx_gfm.GithubFlavoredMarkdownExtension()])
        super(MarkdownWidget, self).__init__(name, *args, **kwargs)
        self.css64 = self.read_file(css_filename)
        self.css64 = self.css64 if self.css64 else css64
        self.css_filename = css_filename

    def read_file(self, filename) -> typing.Optional[str]:
        """
        Find a file named `filename` and return its base64-ecoded content
        """
        data64 = None
        if filename:
            search_paths = list(set(presalytics.autodiscover_paths))
            if os.getcwd() not in search_paths:
                search_paths.append(os.getcwd())
            for path in search_paths:
                fpath = os.path.join(path, filename)
                if os.path.exists(fpath):
                    with open(fpath, 'rb') as f:
                        data = f.read()
                        data64 = base64.b64encode(data).decode('utf-8')  #type: ignore
                    break
            if not data64:
                logger.debug("File {0} could not be found".format(filename))
        return data64


    def to_html(self, data=None, **kwargs) -> str:
        """
        Renders the sandboxed iframe with will contain the d3 script widget
        """
        html = self.md.convert(self.markdown_text)
        return html

    @classmethod
    def deserialize(cls, outline, **kwargs):
        markdown_text = outline.data.get("markdown_text")
        return cls(outline.name,
                   markdown_text=markdown_text,
                   **kwargs)

    def serialize(self, **kwargs):
        data = {
            'markdown_text': self.markdown_text,
        }
        return presalytics.story.outline.Widget(
            name=self.name,
            kind=self.__component_kind__,
            data=data,
        )
    
    @classmethod
    def from_file(cls, name, filepath: str, **kwargs):
        """
        Loads `markdown_text` via a file, rather than a string object.  
        `filepath` can be relative or absolute.
        """
        with open(filepath, 'r') as f:
            markdown_text = f.read()
        return cls(name, markdown_text, **kwargs)
